@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,37 +25,27 @@
 
 #ifdef ENABLE_DATABASE
 #include "db/DatabaseError.hxx"
-#include "db/LightSong.hxx"
 
 #ifdef ENABLE_SQLITE
 #include "sticker/StickerDatabase.hxx"
 #include "sticker/SongSticker.hxx"
 #endif
-
-Database *
-Instance::GetDatabase(Error &error)
-{
-	if (database == nullptr)
-		error.Set(db_domain, DB_DISABLED, "No database");
-	return database;
-}
-
 #endif
 
-void
-Instance::TagModified()
-{
-	partition->TagModified();
-}
-
-void
-Instance::SyncWithPlayer()
-{
-	partition->SyncWithPlayer();
-}
+#include <stdexcept>
 
 #ifdef ENABLE_DATABASE
 
+const Database &
+Instance::GetDatabaseOrThrow() const
+{
+	if (database == nullptr)
+		throw DatabaseError(DatabaseErrorCode::DISABLED,
+				    "No database");
+
+	return *database;
+}
+
 void
 Instance::OnDatabaseModified()
 {
@@ -65,22 +55,24 @@ Instance::OnDatabaseModified()
 
 	stats_invalidate();
 	partition->DatabaseModified(*database);
-	idle_add(IDLE_DATABASE);
 }
 
 void
-Instance::OnDatabaseSongRemoved(const LightSong &song)
+Instance::OnDatabaseSongRemoved(const char *uri)
 {
 	assert(database != nullptr);
 
 #ifdef ENABLE_SQLITE
 	/* if the song has a sticker, remove it */
-	if (sticker_enabled())
-		sticker_song_delete(song);
+	if (sticker_enabled()) {
+		try {
+			sticker_song_delete(uri);
+		} catch (const std::runtime_error &) {
+		}
+	}
 #endif
 
-	const auto uri = song.GetURI();
-	partition->DeleteSong(uri.c_str());
+	partition->StaleSong(uri);
 }
 
 #endif
@@ -90,13 +82,13 @@ Instance::OnDatabaseSongRemoved(const LightSong &song)
 void
 Instance::FoundNeighbor(gcc_unused const NeighborInfo &info)
 {
-	idle_add(IDLE_NEIGHBOR);
+	partition->EmitIdle(IDLE_NEIGHBOR);
 }
 
 void
 Instance::LostNeighbor(gcc_unused const NeighborInfo &info)
 {
-	idle_add(IDLE_NEIGHBOR);
+	partition->EmitIdle(IDLE_NEIGHBOR);
 }
 
 #endif
