@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,13 +19,16 @@
 
 #include "config.h"
 #include "LogV.hxx"
-#include "util/Error.hxx"
+#include "util/Domain.hxx"
+
+#include <exception>
 
-#include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
+static constexpr Domain exception_domain("exception");
+
 void
 LogFormatV(const Domain &domain, LogLevel level, const char *fmt, va_list ap)
 {
@@ -89,20 +92,37 @@ FormatError(const Domain &domain, const char *fmt, ...)
 }
 
 void
-LogError(const Error &error)
+LogError(const std::exception &e)
 {
-	Log(error.GetDomain(), LogLevel::ERROR, error.GetMessage());
+	Log(exception_domain, LogLevel::ERROR, e.what());
+
+	try {
+		std::rethrow_if_nested(e);
+	} catch (const std::exception &nested) {
+		LogError(nested, "nested");
+	} catch (...) {
+		Log(exception_domain, LogLevel::ERROR,
+		    "Unrecognized nested exception");
+	}
 }
 
 void
-LogError(const Error &error, const char *msg)
+LogError(const std::exception &e, const char *msg)
 {
-	LogFormat(error.GetDomain(), LogLevel::ERROR, "%s: %s",
-		  msg, error.GetMessage());
+	FormatError(exception_domain, "%s: %s", msg, e.what());
+
+	try {
+		std::rethrow_if_nested(e);
+	} catch (const std::exception &nested) {
+		LogError(nested);
+	} catch (...) {
+		Log(exception_domain, LogLevel::ERROR,
+		    "Unrecognized nested exception");
+	}
 }
 
 void
-FormatError(const Error &error, const char *fmt, ...)
+FormatError(const std::exception &e, const char *fmt, ...)
 {
 	char msg[1024];
 	va_list ap;
@@ -110,7 +130,45 @@ FormatError(const Error &error, const char *fmt, ...)
 	vsnprintf(msg, sizeof(msg), fmt, ap);
 	va_end(ap);
 
-	LogError(error, msg);
+	LogError(e, msg);
+}
+
+void
+LogError(const std::exception_ptr &ep)
+{
+	try {
+		std::rethrow_exception(ep);
+	} catch (const std::exception &e) {
+		LogError(e);
+	} catch (...) {
+		Log(exception_domain, LogLevel::ERROR,
+		    "Unrecognized exception");
+	}
+}
+
+void
+LogError(const std::exception_ptr &ep, const char *msg)
+{
+	try {
+		std::rethrow_exception(ep);
+	} catch (const std::exception &e) {
+		LogError(e, msg);
+	} catch (...) {
+		FormatError(exception_domain,
+			    "%s: Unrecognized exception", msg);
+	}
+}
+
+void
+FormatError(const std::exception_ptr &ep, const char *fmt, ...)
+{
+	char msg[1024];
+	va_list ap;
+	va_start(ap, fmt);
+	vsnprintf(msg, sizeof(msg), fmt, ap);
+	va_end(ap);
+
+	LogError(ep, msg);
 }
 
 void
