@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,29 +20,26 @@
 #include "config.h"
 #include "MusicBuffer.hxx"
 #include "MusicChunk.hxx"
-#include "system/FatalError.hxx"
 
 #include <assert.h>
 
-MusicBuffer::MusicBuffer(unsigned num_chunks)
+MusicBuffer::MusicBuffer(unsigned num_chunks) noexcept
 	:buffer(num_chunks) {
-	if (buffer.IsOOM())
-		FatalError("Failed to allocate buffer");
 }
 
 MusicChunk *
-MusicBuffer::Allocate()
+MusicBuffer::Allocate() noexcept
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	return buffer.Allocate();
 }
 
 void
-MusicBuffer::Return(MusicChunk *chunk)
+MusicBuffer::Return(MusicChunk *chunk) noexcept
 {
 	assert(chunk != nullptr);
 
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	if (chunk->other != nullptr) {
 		assert(chunk->other->other == nullptr);
