@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 
 #include "config.h"
 #include "Permission.hxx"
-#include "config/ConfigData.hxx"
+#include "config/Param.hxx"
 #include "config/ConfigGlobal.hxx"
 #include "config/ConfigOption.hxx"
 #include "system/FatalError.hxx"
@@ -49,7 +49,6 @@ static std::map<std::string, unsigned> permission_passwords;
 
 static unsigned permission_default;
 
-gcc_pure
 static unsigned
 ParsePermission(const char *p)
 {
@@ -87,12 +86,12 @@ static unsigned parsePermissions(const char *string)
 void initPermissions(void)
 {
 	unsigned permission;
-	const struct config_param *param;
+	const ConfigParam *param;
 
 	permission_default = PERMISSION_READ | PERMISSION_ADD |
 	    PERMISSION_CONTROL | PERMISSION_ADMIN;
 
-	param = config_get_param(CONF_PASSWORD);
+	param = config_get_param(ConfigOption::PASSWORD);
 
 	if (param) {
 		permission_default = 0;
@@ -118,7 +117,7 @@ void initPermissions(void)
 		} while ((param = param->next) != nullptr);
 	}
 
-	param = config_get_param(CONF_DEFAULT_PERMS);
+	param = config_get_param(ConfigOption::DEFAULT_PERMS);
 
 	if (param)
 		permission_default = parsePermissions(param->value.c_str());
