@@ -24,10 +24,12 @@
 #include "tag/Tag.hxx"
 #include "util/ConstBuffer.hxx"
 #include "util/StringAPI.hxx"
+#include "util/StringCompare.hxx"
+#include "util/StringView.hxx"
 #include "util/ASCII.hxx"
 #include "util/TimeParser.hxx"
 #include "util/UriUtil.hxx"
-#include "lib/icu/Collate.hxx"
+#include "lib/icu/CaseFold.hxx"
 
 #include <stdexcept>
 
@@ -57,17 +59,10 @@ locate_parse_type(const char *str) noexcept
 	return tag_name_parse_i(str);
 }
 
-static AllocatedString<>
-ImportString(const char *p, bool fold_case)
-{
-	return fold_case
-		? IcuCaseFold(p)
-		: AllocatedString<>::Duplicate(p);
-}
-
 SongFilter::Item::Item(unsigned _tag, const char *_value, bool _fold_case)
-	:tag(_tag), fold_case(_fold_case),
-	 value(ImportString(_value, _fold_case))
+	:tag(_tag),
+	 value(_value),
+	 fold_case(_fold_case ? IcuCompare(value.c_str()) : IcuCompare())
 {
 }
 
@@ -87,9 +82,7 @@ SongFilter::Item::StringMatch(const char *s) const noexcept
 	assert(tag != LOCATE_TAG_MODIFIED_SINCE);
 
 	if (fold_case) {
-		const auto folded = IcuCaseFold(s);
-		assert(!folded.IsNull());
-		return StringFind(folded.c_str(), value.c_str()) != nullptr;
+		return fold_case.IsIn(s);
 	} else {
 		return StringIsEqual(s, value.c_str());
 	}
@@ -283,3 +276,33 @@ SongFilter::GetBase() const noexcept
 
 	return nullptr;
 }
+
+SongFilter
+SongFilter::WithoutBasePrefix(const char *_prefix) const noexcept
+{
+	const StringView prefix(_prefix);
+	SongFilter result;
+
+	for (const auto &i : items) {
+		if (i.GetTag() == LOCATE_TAG_BASE_TYPE) {
+			const char *s = StringAfterPrefix(i.GetValue(), prefix);
+			if (s != nullptr) {
+				if (*s == 0)
+					continue;
+
+				if (*s == '/') {
+					++s;
+
+					if (*s != 0)
+						result.items.emplace_back(LOCATE_TAG_BASE_TYPE, s);
+
+					continue;
+				}
+			}
+		}
+
+		result.items.emplace_back(i);
+	}
+
+	return result;
+}
