@@ -115,7 +115,12 @@ Iso9660ArchiveFile::Visit(char *path, size_t length, size_t capacity,
 			visitor.VisitArchiveEntry(path + 1);
 		}
 	}
+
+#if LIBCDIO_VERSION_NUM >= 20000
+	iso9660_filelist_free(entlist);
+#else
 	_cdio_list_free (entlist, true);
+#endif
 }
 
 static ArchiveFile *
@@ -182,6 +187,8 @@ Iso9660ArchiveFile::OpenStream(const char *pathname,
 size_t
 Iso9660InputStream::Read(void *ptr, size_t read_size)
 {
+	const ScopeUnlock unlock(mutex);
+
 	int readed = 0;
 	int no_blocks, cur_block;
 	size_t left_bytes = statbuf->size - offset;
