@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@ Client::SetExpired()
 		return;
 
 	FullyBufferedSocket::Close();
-	TimeoutMonitor::Schedule(0);
+	TimeoutMonitor::Schedule(std::chrono::steady_clock::duration::zero());
 }
 
 void
