@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,23 +19,21 @@
 
 #include "config.h"
 #include "OutputCommands.hxx"
+#include "Request.hxx"
 #include "output/OutputPrint.hxx"
 #include "output/OutputCommand.hxx"
-#include "protocol/Result.hxx"
-#include "protocol/ArgParser.hxx"
 #include "client/Client.hxx"
+#include "client/Response.hxx"
 #include "Partition.hxx"
 
 CommandResult
-handle_enableoutput(Client &client, gcc_unused unsigned argc, char *argv[])
+handle_enableoutput(Client &client, Request args, Response &r)
 {
-	unsigned device;
-	if (!check_unsigned(client, &device, argv[1]))
-		return CommandResult::ERROR;
+	assert(args.size == 1);
+	unsigned device = args.ParseUnsigned(0);
 
 	if (!audio_output_enable_index(client.partition.outputs, device)) {
-		command_error(client, ACK_ERROR_NO_EXIST,
-			      "No such audio output");
+		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
 
@@ -43,15 +41,13 @@ handle_enableoutput(Client &client, gcc_unused unsigned argc, char *argv[])
 }
 
 CommandResult
-handle_disableoutput(Client &client, gcc_unused unsigned argc, char *argv[])
+handle_disableoutput(Client &client, Request args, Response &r)
 {
-	unsigned device;
-	if (!check_unsigned(client, &device, argv[1]))
-		return CommandResult::ERROR;
+	assert(args.size == 1);
+	unsigned device = args.ParseUnsigned(0);
 
 	if (!audio_output_disable_index(client.partition.outputs, device)) {
-		command_error(client, ACK_ERROR_NO_EXIST,
-			      "No such audio output");
+		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
 
@@ -59,15 +55,13 @@ handle_disableoutput(Client &client, gcc_unused unsigned argc, char *argv[])
 }
 
 CommandResult
-handle_toggleoutput(Client &client, gcc_unused unsigned argc, char *argv[])
+handle_toggleoutput(Client &client, Request args, Response &r)
 {
-	unsigned device;
-	if (!check_unsigned(client, &device, argv[1]))
-		return CommandResult::ERROR;
+	assert(args.size == 1);
+	unsigned device = args.ParseUnsigned(0);
 
 	if (!audio_output_toggle_index(client.partition.outputs, device)) {
-		command_error(client, ACK_ERROR_NO_EXIST,
-			      "No such audio output");
+		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
 
@@ -75,10 +69,10 @@ handle_toggleoutput(Client &client, gcc_unused unsigned argc, char *argv[])
 }
 
 CommandResult
-handle_devices(Client &client,
-	       gcc_unused unsigned argc, gcc_unused char *argv[])
+handle_devices(Client &client, gcc_unused Request args, Response &r)
 {
-	printAudioDevices(client, client.partition.outputs);
+	assert(args.IsEmpty());
 
+	printAudioDevices(r, client.partition.outputs);
 	return CommandResult::OK;
 }
