@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,17 +20,15 @@
 #include "config.h"
 #include "DatabaseSave.hxx"
 #include "db/DatabaseLock.hxx"
-#include "db/DatabaseError.hxx"
 #include "Directory.hxx"
 #include "DirectorySave.hxx"
 #include "fs/io/BufferedOutputStream.hxx"
 #include "fs/io/TextFile.hxx"
 #include "tag/Tag.hxx"
-#include "tag/TagSettings.h"
+#include "tag/Settings.hxx"
 #include "fs/Charset.hxx"
-#include "util/StringUtil.hxx"
-#include "util/Error.hxx"
-#include "Log.hxx"
+#include "util/StringCompare.hxx"
+#include "util/RuntimeError.hxx"
 
 #include <string.h>
 #include <stdlib.h>
@@ -58,7 +56,7 @@ db_save_internal(BufferedOutputStream &os, const Directory &music_root)
 	os.Format("%s%s\n", DIRECTORY_FS_CHARSET, GetFSCharset());
 
 	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; ++i)
-		if (!ignore_tag_items[i])
+		if (IsTagEnabled(i))
 			os.Format(DB_TAG_PREFIX "%s\n", tag_item_names[i]);
 
 	os.Format("%s\n", DIRECTORY_INFO_END);
@@ -66,95 +64,69 @@ db_save_internal(BufferedOutputStream &os, const Directory &music_root)
 	directory_save(os, music_root);
 }
 
-bool
-db_load_internal(TextFile &file, Directory &music_root, Error &error)
+void
+db_load_internal(TextFile &file, Directory &music_root)
 {
 	char *line;
 	unsigned format = 0;
 	bool found_charset = false, found_version = false;
-	bool success;
 	bool tags[TAG_NUM_OF_ITEM_TYPES];
 
 	/* get initial info */
 	line = file.ReadLine();
-	if (line == nullptr || strcmp(DIRECTORY_INFO_BEGIN, line) != 0) {
-		error.Set(db_domain, "Database corrupted");
-		return false;
-	}
+	if (line == nullptr || strcmp(DIRECTORY_INFO_BEGIN, line) != 0)
+		throw std::runtime_error("Database corrupted");
 
 	memset(tags, false, sizeof(tags));
 
 	while ((line = file.ReadLine()) != nullptr &&
 	       strcmp(line, DIRECTORY_INFO_END) != 0) {
-		if (StringStartsWith(line, DB_FORMAT_PREFIX)) {
-			format = atoi(line + sizeof(DB_FORMAT_PREFIX) - 1);
+		const char *p;
+
+		if ((p = StringAfterPrefix(line, DB_FORMAT_PREFIX))) {
+			format = atoi(p);
 		} else if (StringStartsWith(line, DIRECTORY_MPD_VERSION)) {
-			if (found_version) {
-				error.Set(db_domain, "Duplicate version line");
-				return false;
-			}
+			if (found_version)
+				throw std::runtime_error("Duplicate version line");
 
 			found_version = true;
-		} else if (StringStartsWith(line, DIRECTORY_FS_CHARSET)) {
-			const char *new_charset;
-
-			if (found_charset) {
-				error.Set(db_domain, "Duplicate charset line");
-				return false;
-			}
+		} else if ((p = StringAfterPrefix(line, DIRECTORY_FS_CHARSET))) {
+			if (found_charset)
+				throw std::runtime_error("Duplicate charset line");
 
 			found_charset = true;
 
-			new_charset = line + sizeof(DIRECTORY_FS_CHARSET) - 1;
+			const char *new_charset = p;
 			const char *const old_charset = GetFSCharset();
 			if (*old_charset != 0
-			    && strcmp(new_charset, old_charset) != 0) {
-				error.Format(db_domain,
-					     "Existing database has charset "
-					     "\"%s\" instead of \"%s\"; "
-					     "discarding database file",
-					     new_charset, old_charset);
-				return false;
-			}
-		} else if (StringStartsWith(line, DB_TAG_PREFIX)) {
-			const char *name = line + sizeof(DB_TAG_PREFIX) - 1;
+			    && strcmp(new_charset, old_charset) != 0)
+				throw FormatRuntimeError("Existing database has charset "
+							 "\"%s\" instead of \"%s\"; "
+							 "discarding database file",
+							 new_charset, old_charset);
+		} else if ((p = StringAfterPrefix(line, DB_TAG_PREFIX))) {
+			const char *name = p;
 			TagType tag = tag_name_parse(name);
-			if (tag == TAG_NUM_OF_ITEM_TYPES) {
-				error.Format(db_domain,
-					     "Unrecognized tag '%s', "
-					     "discarding database file",
-					     name);
-				return false;
-			}
+			if (tag == TAG_NUM_OF_ITEM_TYPES)
+				throw FormatRuntimeError("Unrecognized tag '%s', "
+							 "discarding database file",
+							 name);
 
 			tags[tag] = true;
 		} else {
-			error.Format(db_domain, "Malformed line: %s", line);
-			return false;
+			throw FormatRuntimeError("Malformed line: %s", line);
 		}
 	}
 
-	if (format < OLDEST_DB_FORMAT || format > DB_FORMAT) {
-		error.Set(db_domain,
-			  "Database format mismatch, "
-			  "discarding database file");
-		return false;
-	}
+	if (format < OLDEST_DB_FORMAT || format > DB_FORMAT)
+		throw std::runtime_error("Database format mismatch, "
+					 "discarding database file");
 
-	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; ++i) {
-		if (!ignore_tag_items[i] && !tags[i]) {
-			error.Set(db_domain,
-				  "Tag list mismatch, "
-				  "discarding database file");
-			return false;
-		}
-	}
+	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; ++i)
+		if (IsTagEnabled(i) && !tags[i])
+			throw std::runtime_error("Tag list mismatch, "
+						 "discarding database file");
 
-	LogDebug(db_domain, "reading DB");
-
-	db_lock();
-	success = directory_load(file, music_root, error);
-	db_unlock();
-
-	return success;
+	const ScopeDatabaseLock protect;
+	directory_load(file, music_root);
 }
