@@ -20,18 +20,12 @@
 #include "config.h"
 #include "Mount.hxx"
 #include "PrefixedLightSong.hxx"
+#include "SongFilter.hxx"
 #include "db/Selection.hxx"
 #include "db/LightDirectory.hxx"
 #include "db/Interface.hxx"
 #include "fs/Traits.hxx"
 
-#ifdef _LIBCPP_VERSION
-/* workaround for "error: incomplete type 'PlaylistInfo' used in type
-   trait expression" with libc++ version 3900 (from Android NDK
-   r13b) */
-#include "db/PlaylistInfo.hxx"
-#endif
-
 #include <string>
 
 struct PrefixedLightDirectory : LightDirectory {
@@ -72,7 +66,7 @@ PrefixVisitPlaylist(const char *base, const VisitPlaylist &visit_playlist,
 
 void
 WalkMount(const char *base, const Database &db,
-	  bool recursive, const SongFilter *filter,
+	  const char* uri, bool recursive, const SongFilter *filter,
 	  const VisitDirectory &visit_directory, const VisitSong &visit_song,
 	  const VisitPlaylist &visit_playlist)
 {
@@ -93,5 +87,16 @@ WalkMount(const char *base, const Database &db,
 		vp = std::bind(PrefixVisitPlaylist,
 			       base, std::ref(visit_playlist), _1, _2);
 
-	db.Visit(DatabaseSelection("", recursive, filter), vd, vs, vp);
+	SongFilter prefix_filter;
+
+	if (base != nullptr && filter != nullptr) {
+		/* if the SongFilter contains a LOCATE_TAG_BASE_TYPE
+		   item, copy the SongFilter and drop the mount point
+		   from the filter, because the mounted database
+		   doesn't know its own location within MPD's VFS */
+		prefix_filter = filter->WithoutBasePrefix(base);
+		filter = &prefix_filter;
+	}
+
+	db.Visit(DatabaseSelection(uri, recursive, filter), vd, vs, vp);
 }
