@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,15 +22,15 @@
 #include "InotifyDomain.hxx"
 #include "Service.hxx"
 #include "Log.hxx"
-
-#include <string.h>
+#include "util/StringCompare.hxx"
 
 /**
  * Wait this long after the last change before calling
  * update_enqueue().  This increases the probability that updates can
  * be bundled.
  */
-static constexpr unsigned INOTIFY_UPDATE_DELAY_S = 5;
+static constexpr std::chrono::steady_clock::duration INOTIFY_UPDATE_DELAY =
+	std::chrono::seconds(5);
 
 void
 InotifyQueue::OnTimeout()
@@ -43,7 +43,7 @@ InotifyQueue::OnTimeout()
 		id = update.Enqueue(uri_utf8, false);
 		if (id == 0) {
 			/* retry later */
-			ScheduleSeconds(INOTIFY_UPDATE_DELAY_S);
+			Schedule(INOTIFY_UPDATE_DELAY);
 			return;
 		}
 
@@ -54,20 +54,22 @@ InotifyQueue::OnTimeout()
 	}
 }
 
+gcc_pure
 static bool
-path_in(const char *path, const char *possible_parent)
+path_in(const char *path, const char *possible_parent) noexcept
 {
-	size_t length = strlen(possible_parent);
+	if (StringIsEmpty(path))
+		return true;
 
-	return path[0] == 0 ||
-		(memcmp(possible_parent, path, length) == 0 &&
-		 (path[length] == 0 || path[length] == '/'));
+	auto rest = StringAfterPrefix(path, possible_parent);
+	return rest != nullptr &&
+		(StringIsEmpty(rest) || rest[0] == '/');
 }
 
 void
 InotifyQueue::Enqueue(const char *uri_utf8)
 {
-	ScheduleSeconds(INOTIFY_UPDATE_DELAY_S);
+	Schedule(INOTIFY_UPDATE_DELAY);
 
 	for (auto i = queue.begin(), end = queue.end(); i != end;) {
 		const char *current_uri = i->c_str();
