@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,8 +24,7 @@
 #include "InotifyDomain.hxx"
 #include "storage/StorageInterface.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "fs/FileSystem.hxx"
-#include "util/Error.hxx"
+#include "fs/FileInfo.hxx"
 #include "Log.hxx"
 
 #include <string>
@@ -34,7 +33,6 @@
 
 #include <assert.h>
 #include <sys/inotify.h>
-#include <sys/stat.h>
 #include <string.h>
 #include <dirent.h>
 
@@ -64,10 +62,10 @@ struct WatchDirectory {
 	WatchDirectory &operator=(const WatchDirectory &) = delete;
 
 	gcc_pure
-	unsigned GetDepth() const;
+	unsigned GetDepth() const noexcept;
 
 	gcc_pure
-	AllocatedPath GetUriFS() const;
+	AllocatedPath GetUriFS() const noexcept;
 };
 
 static InotifySource *inotify_source;
@@ -134,7 +132,7 @@ remove_watch_directory(WatchDirectory *directory)
 }
 
 AllocatedPath
-WatchDirectory::GetUriFS() const
+WatchDirectory::GetUriFS() const noexcept
 {
 	if (parent == nullptr)
 		return AllocatedPath::Null();
@@ -158,7 +156,6 @@ static void
 recursive_watch_subdirectories(WatchDirectory *directory,
 			       const AllocatedPath &path_fs, unsigned depth)
 {
-	Error error;
 	DIR *dir;
 	struct dirent *ent;
 
@@ -179,7 +176,6 @@ recursive_watch_subdirectories(WatchDirectory *directory,
 	}
 
 	while ((ent = readdir(dir))) {
-		struct stat st;
 		int ret;
 
 		if (skip_path(ent->d_name))
@@ -187,24 +183,25 @@ recursive_watch_subdirectories(WatchDirectory *directory,
 
 		const auto child_path_fs =
 			AllocatedPath::Build(path_fs, ent->d_name);
-		ret = StatFile(child_path_fs, st);
-		if (ret < 0) {
-			FormatErrno(inotify_domain,
-				    "Failed to stat %s",
-				    child_path_fs.c_str());
+
+		FileInfo fi;
+		try {
+			fi = FileInfo(child_path_fs);
+		} catch (const std::runtime_error &e) {
+			LogError(e);
 			continue;
 		}
 
-		if (!S_ISDIR(st.st_mode))
+		if (!fi.IsDirectory())
 			continue;
 
-		ret = inotify_source->Add(child_path_fs.c_str(), IN_MASK,
-					  error);
-		if (ret < 0) {
-			FormatError(error,
+		try {
+			ret = inotify_source->Add(child_path_fs.c_str(),
+						  IN_MASK);
+		} catch (const std::runtime_error &e) {
+			FormatError(e,
 				    "Failed to register %s",
 				    child_path_fs.c_str());
-			error.Clear();
 			continue;
 		}
 
@@ -228,7 +225,7 @@ recursive_watch_subdirectories(WatchDirectory *directory,
 
 gcc_pure
 unsigned
-WatchDirectory::GetDepth() const
+WatchDirectory::GetDepth() const noexcept
 {
 	const WatchDirectory *d = this;
 	unsigned depth = 0;
@@ -301,20 +298,22 @@ mpd_inotify_init(EventLoop &loop, Storage &storage, UpdateService &update,
 		return;
 	}
 
-	Error error;
-	inotify_source = InotifySource::Create(loop,
-					       mpd_inotify_callback, nullptr,
-					       error);
-	if (inotify_source == nullptr) {
-		LogError(error);
+	try {
+		inotify_source = new InotifySource(loop,
+						   mpd_inotify_callback,
+						   nullptr);
+	} catch (const std::runtime_error &e) {
+		LogError(e);
 		return;
 	}
 
 	inotify_max_depth = max_depth;
 
-	int descriptor = inotify_source->Add(path.c_str(), IN_MASK, error);
-	if (descriptor < 0) {
-		LogError(error);
+	int descriptor;
+	try {
+		descriptor = inotify_source->Add(path.c_str(), IN_MASK);
+	} catch (const std::runtime_error &e) {
+		LogError(e);
 		delete inotify_source;
 		inotify_source = nullptr;
 		return;
@@ -332,7 +331,7 @@ mpd_inotify_init(EventLoop &loop, Storage &storage, UpdateService &update,
 }
 
 void
-mpd_inotify_finish(void)
+mpd_inotify_finish(void) noexcept
 {
 	if (inotify_source == nullptr)
 		return;
