@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,13 @@
 
 #include "config.h" /* must be first for large file support */
 #include "FlacDecoderPlugin.h"
+#include "FlacStreamDecoder.hxx"
 #include "FlacDomain.hxx"
 #include "FlacCommon.hxx"
 #include "FlacMetadata.hxx"
 #include "OggCodec.hxx"
 #include "fs/Path.hxx"
-#include "util/Error.hxx"
+#include "fs/NarrowPath.hxx"
 #include "Log.hxx"
 
 #if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
@@ -55,36 +56,24 @@ static void flacPrintErroredState(FLAC__StreamDecoderState state)
 static void flacMetadata(gcc_unused const FLAC__StreamDecoder * dec,
 			 const FLAC__StreamMetadata * block, void *vdata)
 {
-	flac_metadata_common_cb(block, (struct flac_data *) vdata);
+	auto &fd = *(FlacDecoder *)vdata;
+	fd.OnMetadata(*block);
 }
 
 static FLAC__StreamDecoderWriteStatus
 flac_write_cb(const FLAC__StreamDecoder *dec, const FLAC__Frame *frame,
 	      const FLAC__int32 *const buf[], void *vdata)
 {
-	struct flac_data *data = (struct flac_data *) vdata;
-	FLAC__uint64 nbytes = 0;
-
-	if (FLAC__stream_decoder_get_decode_position(dec, &nbytes)) {
-		if (data->position > 0 && nbytes > data->position) {
-			nbytes -= data->position;
-			data->position += nbytes;
-		} else {
-			data->position = nbytes;
-			nbytes = 0;
-		}
-	} else
-		nbytes = 0;
-
-	return flac_common_write(data, frame, buf, nbytes);
+	auto &fd = *(FlacDecoder *)vdata;
+	return fd.OnWrite(*frame, buf, fd.GetDeltaPosition(*dec));
 }
 
 static bool
 flac_scan_file(Path path_fs,
-	       const struct tag_handler *handler, void *handler_ctx)
+	       const TagHandler &handler, void *handler_ctx)
 {
 	FlacMetadataChain chain;
-	if (!chain.Read(path_fs.c_str())) {
+	if (!chain.Read(NarrowPath(path_fs))) {
 		FormatDebug(flac_domain,
 			    "Failed to read FLAC tags: %s",
 			    chain.GetStatusString());
@@ -97,7 +86,7 @@ flac_scan_file(Path path_fs,
 
 static bool
 flac_scan_stream(InputStream &is,
-		 const struct tag_handler *handler, void *handler_ctx)
+		 const TagHandler &handler, void *handler_ctx)
 {
 	FlacMetadataChain chain;
 	if (!chain.Read(is)) {
@@ -114,17 +103,11 @@ flac_scan_stream(InputStream &is,
 /**
  * Some glue code around FLAC__stream_decoder_new().
  */
-static FLAC__StreamDecoder *
+static FlacStreamDecoder
 flac_decoder_new(void)
 {
-	FLAC__StreamDecoder *sd = FLAC__stream_decoder_new();
-	if (sd == nullptr) {
-		LogError(flac_domain,
-			 "FLAC__stream_decoder_new() failed");
-		return nullptr;
-	}
-
-	if(!FLAC__stream_decoder_set_metadata_respond(sd, FLAC__METADATA_TYPE_VORBIS_COMMENT))
+	FlacStreamDecoder sd;
+	if(!FLAC__stream_decoder_set_metadata_respond(sd.get(), FLAC__METADATA_TYPE_VORBIS_COMMENT))
 		LogDebug(flac_domain,
 			 "FLAC__stream_decoder_set_metadata_respond() has failed");
 
@@ -132,7 +115,7 @@ flac_decoder_new(void)
 }
 
 static bool
-flac_decoder_initialize(struct flac_data *data, FLAC__StreamDecoder *sd)
+flac_decoder_initialize(FlacDecoder *data, FLAC__StreamDecoder *sd)
 {
 	if (!FLAC__stream_decoder_process_until_end_of_metadata(sd)) {
 		if (FLAC__stream_decoder_get_state(sd) != FLAC__STREAM_DECODER_END_OF_STREAM)
@@ -145,7 +128,7 @@ flac_decoder_initialize(struct flac_data *data, FLAC__StreamDecoder *sd)
 		return true;
 	}
 
-	if (data->input_stream.IsSeekable())
+	if (data->GetInputStream().IsSeekable())
 		/* allow the workaround below only for nonseekable
 		   streams*/
 		return false;
@@ -156,28 +139,53 @@ flac_decoder_initialize(struct flac_data *data, FLAC__StreamDecoder *sd)
 	return data->initialized;
 }
 
-static void
-flac_decoder_loop(struct flac_data *data, FLAC__StreamDecoder *flac_dec)
+static DecoderCommand
+FlacSubmitToClient(DecoderClient &client, FlacDecoder &d) noexcept
 {
-	Decoder &decoder = data->decoder;
+	if (d.tag.IsEmpty() && d.chunk.IsEmpty())
+		return client.GetCommand();
+
+	if (!d.tag.IsEmpty()) {
+		auto cmd = client.SubmitTag(d.GetInputStream(),
+					    std::move(d.tag));
+		d.tag.Clear();
+		if (cmd != DecoderCommand::NONE)
+			return cmd;
+	}
+
+	if (!d.chunk.IsEmpty()) {
+		auto cmd = client.SubmitData(d.GetInputStream(),
+					     d.chunk.data,
+					     d.chunk.size,
+					     d.kbit_rate);
+		d.chunk = nullptr;
+		if (cmd != DecoderCommand::NONE)
+			return cmd;
+	}
+
+	return DecoderCommand::NONE;
+}
+
+static void
+flac_decoder_loop(FlacDecoder *data, FLAC__StreamDecoder *flac_dec)
+{
+	DecoderClient &client = *data->GetClient();
 
 	while (true) {
-		DecoderCommand cmd;
-		if (!data->tag.IsEmpty()) {
-			cmd = decoder_tag(data->decoder, data->input_stream,
-					  std::move(data->tag));
-			data->tag.Clear();
-		} else
-			cmd = decoder_get_command(decoder);
+		DecoderCommand cmd = FlacSubmitToClient(client, *data);
 
 		if (cmd == DecoderCommand::SEEK) {
-			FLAC__uint64 seek_sample =
-				decoder_seek_where_frame(decoder);
+			FLAC__uint64 seek_sample = client.GetSeekFrame();
 			if (FLAC__stream_decoder_seek_absolute(flac_dec, seek_sample)) {
 				data->position = 0;
-				decoder_command_finished(decoder);
+				client.CommandFinished();
 			} else
-				decoder_seek_error(decoder);
+				client.SeekError();
+
+			/* FLAC__stream_decoder_seek_absolute()
+			   decodes one frame and may have provided
+			   data to be submitted to the client */
+			continue;
 		} else if (cmd == DecoderCommand::STOP)
 			break;
 
@@ -215,7 +223,7 @@ flac_decoder_loop(struct flac_data *data, FLAC__StreamDecoder *flac_dec)
 		}
 
 		if (!FLAC__stream_decoder_process_single(flac_dec) &&
-		    decoder_get_command(decoder) == DecoderCommand::NONE) {
+		    client.GetCommand() == DecoderCommand::NONE) {
 			/* a failure that was not triggered by a
 			   decoder command */
 			flacPrintErroredState(FLAC__stream_decoder_get_state(flac_dec));
@@ -225,7 +233,7 @@ flac_decoder_loop(struct flac_data *data, FLAC__StreamDecoder *flac_dec)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init_oggflac(FLAC__StreamDecoder *flac_dec, struct flac_data *data)
+stream_init_oggflac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
 {
 	return FLAC__stream_decoder_init_ogg_stream(flac_dec,
 						    FlacInput::Read,
@@ -240,7 +248,7 @@ stream_init_oggflac(FLAC__StreamDecoder *flac_dec, struct flac_data *data)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init_flac(FLAC__StreamDecoder *flac_dec, struct flac_data *data)
+stream_init_flac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
 {
 	return FLAC__stream_decoder_init_stream(flac_dec,
 						FlacInput::Read,
@@ -255,7 +263,7 @@ stream_init_flac(FLAC__StreamDecoder *flac_dec, struct flac_data *data)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init(FLAC__StreamDecoder *flac_dec, struct flac_data *data, bool is_ogg)
+stream_init(FLAC__StreamDecoder *flac_dec, FlacDecoder *data, bool is_ogg)
 {
 	return is_ogg
 		? stream_init_oggflac(flac_dec, data)
@@ -263,7 +271,7 @@ stream_init(FLAC__StreamDecoder *flac_dec, struct flac_data *data, bool is_ogg)
 }
 
 static bool
-FlacInitAndDecode(struct flac_data &data, FLAC__StreamDecoder *sd, bool is_ogg)
+FlacInitAndDecode(FlacDecoder &data, FLAC__StreamDecoder *sd, bool is_ogg)
 {
 	auto init_status = stream_init(sd, &data, is_ogg);
 	if (init_status != FLAC__STREAM_DECODER_INIT_STATUS_OK) {
@@ -281,41 +289,37 @@ FlacInitAndDecode(struct flac_data &data, FLAC__StreamDecoder *sd, bool is_ogg)
 }
 
 static void
-flac_decode_internal(Decoder &decoder,
+flac_decode_internal(DecoderClient &client,
 		     InputStream &input_stream,
 		     bool is_ogg)
 {
-	FLAC__StreamDecoder *flac_dec;
-
-	flac_dec = flac_decoder_new();
-	if (flac_dec == nullptr)
+	auto flac_dec = flac_decoder_new();
+	if (!flac_dec)
 		return;
 
-	struct flac_data data(decoder, input_stream);
+	FlacDecoder data(client, input_stream);
 
-	FlacInitAndDecode(data, flac_dec, is_ogg);
-
-	FLAC__stream_decoder_delete(flac_dec);
+	FlacInitAndDecode(data, flac_dec.get(), is_ogg);
 }
 
 static void
-flac_decode(Decoder &decoder, InputStream &input_stream)
+flac_decode(DecoderClient &client, InputStream &input_stream)
 {
-	flac_decode_internal(decoder, input_stream, false);
+	flac_decode_internal(client, input_stream, false);
 }
 
 static bool
-oggflac_init(gcc_unused const config_param &param)
+oggflac_init(gcc_unused const ConfigBlock &block)
 {
 	return !!FLAC_API_SUPPORTS_OGG_FLAC;
 }
 
 static bool
 oggflac_scan_file(Path path_fs,
-		  const struct tag_handler *handler, void *handler_ctx)
+		  const TagHandler &handler, void *handler_ctx)
 {
 	FlacMetadataChain chain;
-	if (!chain.ReadOgg(path_fs.c_str())) {
+	if (!chain.ReadOgg(NarrowPath(path_fs))) {
 		FormatDebug(flac_domain,
 			    "Failed to read OggFLAC tags: %s",
 			    chain.GetStatusString());
@@ -328,7 +332,7 @@ oggflac_scan_file(Path path_fs,
 
 static bool
 oggflac_scan_stream(InputStream &is,
-		    const struct tag_handler *handler, void *handler_ctx)
+		    const TagHandler &handler, void *handler_ctx)
 {
 	FlacMetadataChain chain;
 	if (!chain.ReadOgg(is)) {
@@ -343,16 +347,19 @@ oggflac_scan_stream(InputStream &is,
 }
 
 static void
-oggflac_decode(Decoder &decoder, InputStream &input_stream)
+oggflac_decode(DecoderClient &client, InputStream &input_stream)
 {
-	if (ogg_codec_detect(&decoder, input_stream) != OGG_CODEC_FLAC)
+	if (ogg_codec_detect(&client, input_stream) != OGG_CODEC_FLAC)
 		return;
 
 	/* rewind the stream, because ogg_codec_detect() has
 	   moved it */
-	input_stream.LockRewind(IgnoreError());
+	try {
+		input_stream.LockRewind();
+	} catch (const std::runtime_error &) {
+	}
 
-	flac_decode_internal(decoder, input_stream, true);
+	flac_decode_internal(client, input_stream, true);
 }
 
 static const char *const oggflac_suffixes[] = { "ogg", "oga", nullptr };
