@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,10 @@
 #include "WildmidiDecoderPlugin.hxx"
 #include "../DecoderAPI.hxx"
 #include "tag/TagHandler.hxx"
-#include "util/Error.hxx"
 #include "util/Domain.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/FileSystem.hxx"
 #include "fs/Path.hxx"
-#include "system/FatalError.hxx"
 #include "Log.hxx"
 
 extern "C" {
@@ -38,15 +36,11 @@ static constexpr Domain wildmidi_domain("wildmidi");
 static constexpr unsigned WILDMIDI_SAMPLE_RATE = 48000;
 
 static bool
-wildmidi_init(const config_param &param)
+wildmidi_init(const ConfigBlock &block)
 {
-	Error error;
 	const AllocatedPath path =
-		param.GetBlockPath("config_file",
-				   "/etc/timidity/timidity.cfg",
-				   error);
-	if (path.IsNull())
-		FatalError(error);
+		block.GetPath("config_file",
+			      "/etc/timidity/timidity.cfg");
 
 	if (!FileExists(path)) {
 		const auto utf8 = path.ToUTF8();
@@ -66,7 +60,7 @@ wildmidi_finish(void)
 }
 
 static DecoderCommand
-wildmidi_output(Decoder &decoder, midi *wm)
+wildmidi_output(DecoderClient &client, midi *wm)
 {
 #ifdef LIBWILDMIDI_VER_MAJOR
 	/* WildMidi 0.4 has switched from "char*" to "int8_t*" */
@@ -80,11 +74,11 @@ wildmidi_output(Decoder &decoder, midi *wm)
 	if (length <= 0)
 		return DecoderCommand::STOP;
 
-	return decoder_data(decoder, nullptr, buffer, length, 0);
+	return client.SubmitData(nullptr, buffer, length, 0);
 }
 
 static void
-wildmidi_file_decode(Decoder &decoder, Path path_fs)
+wildmidi_file_decode(DecoderClient &client, Path path_fs)
 {
 	static constexpr AudioFormat audio_format = {
 		WILDMIDI_SAMPLE_RATE,
@@ -108,7 +102,7 @@ wildmidi_file_decode(Decoder &decoder, Path path_fs)
 		SongTime::FromScale<uint64_t>(info->approx_total_samples,
 					      WILDMIDI_SAMPLE_RATE);
 
-	decoder_initialized(decoder, audio_format, true, duration);
+	client.Ready(audio_format, true, duration);
 
 	DecoderCommand cmd;
 	do {
@@ -116,14 +110,13 @@ wildmidi_file_decode(Decoder &decoder, Path path_fs)
 		if (info == nullptr)
 			break;
 
-		cmd = wildmidi_output(decoder, wm);
+		cmd = wildmidi_output(client, wm);
 
 		if (cmd == DecoderCommand::SEEK) {
-			unsigned long seek_where =
-				decoder_seek_where_frame(decoder);
+			unsigned long seek_where = client.GetSeekFrame();
 
 			WildMidi_FastSeek(wm, &seek_where);
-			decoder_command_finished(decoder);
+			client.CommandFinished();
 			cmd = DecoderCommand::NONE;
 		}
 
@@ -134,7 +127,7 @@ wildmidi_file_decode(Decoder &decoder, Path path_fs)
 
 static bool
 wildmidi_scan_file(Path path_fs,
-		   const struct tag_handler *handler, void *handler_ctx)
+		   const TagHandler &handler, void *handler_ctx)
 {
 	midi *wm = WildMidi_Open(path_fs.c_str());
 	if (wm == nullptr)
