@@ -23,7 +23,7 @@
 
 #include <algorithm>
 
-#ifndef WIN32
+#ifndef _WIN32
 #include <poll.h>
 #endif
 
@@ -50,7 +50,7 @@ MultiSocketMonitor::ClearSocketList()
 	fds.clear();
 }
 
-#ifndef WIN32
+#ifndef _WIN32
 
 void
 MultiSocketMonitor::ReplaceSocketList(pollfd *pfds, unsigned n)
