@@ -73,6 +73,10 @@ public:
 		return filter;
 	}
 
+	void Reset() override {
+		filter->Reset();
+	}
+
 	ConstBuffer<void> FilterPCM(ConstBuffer<void> src) override {
 		return filter->FilterPCM(src);
 	}
