@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,19 +21,21 @@
 #include "Init.hxx"
 #include "Registry.hxx"
 #include "InputPlugin.hxx"
-#include "util/Error.hxx"
 #include "config/ConfigGlobal.hxx"
 #include "config/ConfigOption.hxx"
-#include "config/ConfigData.hxx"
+#include "config/Block.hxx"
 #include "Log.hxx"
+#include "PluginUnavailable.hxx"
+#include "util/RuntimeError.hxx"
+
+#include <stdexcept>
 
 #include <assert.h>
-#include <string.h>
 
-bool
-input_stream_global_init(Error &error)
+void
+input_stream_global_init()
 {
-	const config_param empty;
+	const ConfigBlock empty;
 
 	for (unsigned i = 0; input_plugins[i] != nullptr; ++i) {
 		const InputPlugin *plugin = input_plugins[i];
@@ -42,41 +44,29 @@ input_stream_global_init(Error &error)
 		assert(*plugin->name != 0);
 		assert(plugin->open != nullptr);
 
-		const struct config_param *param =
-			config_find_block(CONF_INPUT, "plugin", plugin->name);
-		if (param == nullptr) {
-			param = &empty;
-		} else if (!param->GetBlockValue("enabled", true))
+		const auto *block =
+			config_find_block(ConfigBlockOption::INPUT, "plugin",
+					  plugin->name);
+		if (block == nullptr) {
+			block = &empty;
+		} else if (!block->GetBlockValue("enabled", true))
 			/* the plugin is disabled in mpd.conf */
 			continue;
 
-		InputPlugin::InitResult result = plugin->init != nullptr
-			? plugin->init(*param, error)
-			: InputPlugin::InitResult::SUCCESS;
-
-		switch (result) {
-		case InputPlugin::InitResult::SUCCESS:
+		try {
+			if (plugin->init != nullptr)
+				plugin->init(*block);
 			input_plugins_enabled[i] = true;
-			break;
-
-		case InputPlugin::InitResult::ERROR:
-			error.FormatPrefix("Failed to initialize input plugin '%s': ",
-					   plugin->name);
-			return false;
-
-		case InputPlugin::InitResult::UNAVAILABLE:
-			if (error.IsDefined()) {
-				FormatError(error,
-					    "Input plugin '%s' is unavailable",
-					    plugin->name);
-				error.Clear();
-			}
-
-			break;
+		} catch (const PluginUnavailable &e) {
+			FormatError(e,
+				    "Input plugin '%s' is unavailable",
+				    plugin->name);
+			continue;
+		} catch (const std::runtime_error &e) {
+			std::throw_with_nested(FormatRuntimeError("Failed to initialize input plugin '%s'",
+								  plugin->name));
 		}
 	}
-
-	return true;
 }
 
 void input_stream_global_finish(void)
