@@ -34,7 +34,7 @@
 
 #include <algorithm>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <ws2tcpip.h>
 #else
 #include <netdb.h>
