@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 void
 notify::Wait()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	while (!pending)
 		cond.wait(mutex);
 	pending = false;
@@ -32,7 +32,7 @@ notify::Wait()
 void
 notify::Signal()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	pending = true;
 	cond.signal();
 }
@@ -40,6 +40,6 @@ notify::Signal()
 void
 notify::Clear()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	pending = false;
 }
