@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,86 +19,85 @@
 
 #include "config.h"
 #include "MultipleOutputs.hxx"
-#include "PlayerControl.hxx"
 #include "Internal.hxx"
 #include "Domain.hxx"
 #include "MusicBuffer.hxx"
 #include "MusicPipe.hxx"
 #include "MusicChunk.hxx"
-#include "system/FatalError.hxx"
-#include "util/Error.hxx"
-#include "config/ConfigData.hxx"
+#include "config/Block.hxx"
 #include "config/ConfigGlobal.hxx"
 #include "config/ConfigOption.hxx"
 #include "notify.hxx"
+#include "util/RuntimeError.hxx"
+
+#include <stdexcept>
 
 #include <assert.h>
 #include <string.h>
 
 MultipleOutputs::MultipleOutputs(MixerListener &_mixer_listener)
-	:mixer_listener(_mixer_listener),
-	 input_audio_format(AudioFormat::Undefined()),
-	 buffer(nullptr), pipe(nullptr),
-	 elapsed_time(SignedSongTime::Negative())
+	:mixer_listener(_mixer_listener)
 {
 }
 
 MultipleOutputs::~MultipleOutputs()
 {
-	for (auto i : outputs) {
-		i->LockDisableWait();
-		i->Finish();
-	}
+	/* parallel destruction */
+	for (auto i : outputs)
+		i->BeginDestroy();
+	for (auto i : outputs)
+		i->FinishDestroy();
 }
 
 static AudioOutput *
-LoadOutput(EventLoop &event_loop, MixerListener &mixer_listener,
-	   PlayerControl &pc, const config_param &param)
-{
-	Error error;
-	AudioOutput *output = audio_output_new(event_loop, param,
-					       mixer_listener,
-					       pc, error);
-	if (output == nullptr) {
-		if (param.line > 0)
-			FormatFatalError("line %i: %s",
-					 param.line,
-					 error.GetMessage());
-		else
-			FatalError(error);
-	}
-
-	return output;
+LoadOutput(EventLoop &event_loop,
+	   const ReplayGainConfig &replay_gain_config,
+	   MixerListener &mixer_listener,
+	   AudioOutputClient &client, const ConfigBlock &block)
+try {
+	return audio_output_new(event_loop, replay_gain_config, block,
+				mixer_listener,
+				client);
+} catch (const std::runtime_error &e) {
+	if (block.line > 0)
+		std::throw_with_nested(FormatRuntimeError("Failed to configure output in line %i",
+							  block.line));
+	else
+		throw;
 }
 
 void
-MultipleOutputs::Configure(EventLoop &event_loop, PlayerControl &pc)
+MultipleOutputs::Configure(EventLoop &event_loop,
+			   const ReplayGainConfig &replay_gain_config,
+			   AudioOutputClient &client)
 {
-	for (const config_param *param = config_get_param(CONF_AUDIO_OUTPUT);
+	for (const auto *param = config_get_block(ConfigBlockOption::AUDIO_OUTPUT);
 	     param != nullptr; param = param->next) {
-		auto output = LoadOutput(event_loop, mixer_listener,
-					 pc, *param);
-		if (FindByName(output->name) != nullptr)
-			FormatFatalError("output devices with identical "
-					 "names: %s", output->name);
+		auto output = LoadOutput(event_loop, replay_gain_config,
+					 mixer_listener,
+					 client, *param);
+		if (FindByName(output->GetName()) != nullptr)
+			throw FormatRuntimeError("output devices with identical "
+						 "names: %s", output->GetName());
 
 		outputs.push_back(output);
 	}
 
 	if (outputs.empty()) {
 		/* auto-detect device */
-		const config_param empty;
-		auto output = LoadOutput(event_loop, mixer_listener,
-					 pc, empty);
+		const ConfigBlock empty;
+		auto output = LoadOutput(event_loop, replay_gain_config,
+					 mixer_listener,
+					 client, empty);
 		outputs.push_back(output);
 	}
 }
 
 AudioOutput *
-MultipleOutputs::FindByName(const char *name) const
+MultipleOutputs::FindByName(const char *name) const noexcept
 {
 	for (auto i : outputs)
-		if (strcmp(i->name, name) == 0)
+		if (strcmp(i->GetName(), name) == 0)
 			return i;
 
 	return nullptr;
@@ -107,27 +106,24 @@ MultipleOutputs::FindByName(const char *name) const
 void
 MultipleOutputs::EnableDisable()
 {
+	/* parallel execution */
+
 	for (auto ao : outputs) {
-		bool enabled;
+		const std::lock_guard<Mutex> lock(ao->mutex);
+		ao->EnableDisableAsync();
+	}
 
-		ao->mutex.lock();
-		enabled = ao->really_enabled;
-		ao->mutex.unlock();
-
-		if (ao->enabled != enabled) {
-			if (ao->enabled)
-				ao->LockEnableWait();
-			else
-				ao->LockDisableWait();
-		}
+	for (auto ao : outputs) {
+		const std::lock_guard<Mutex> lock(ao->mutex);
+		ao->WaitForCommand();
 	}
 }
 
 bool
-MultipleOutputs::AllFinished() const
+MultipleOutputs::AllFinished() const noexcept
 {
 	for (auto ao : outputs) {
-		const ScopeLock protect(ao->mutex);
+		const std::lock_guard<Mutex> protect(ao->mutex);
 		if (ao->IsOpen() && !ao->IsCommandFinished())
 			return false;
 	}
@@ -136,7 +132,7 @@ MultipleOutputs::AllFinished() const
 }
 
 void
-MultipleOutputs::WaitAll()
+MultipleOutputs::WaitAll() noexcept
 {
 	while (!AllFinished())
 		audio_output_client_notify.Wait();
@@ -149,23 +145,8 @@ MultipleOutputs::AllowPlay()
 		ao->LockAllowPlay();
 }
 
-static void
-audio_output_reset_reopen(AudioOutput *ao)
-{
-	const ScopeLock protect(ao->mutex);
-
-	ao->fail_timer.Reset();
-}
-
-void
-MultipleOutputs::ResetReopen()
-{
-	for (auto ao : outputs)
-		audio_output_reset_reopen(ao);
-}
-
 bool
-MultipleOutputs::Update()
+MultipleOutputs::Update(bool force)
 {
 	bool ret = false;
 
@@ -173,7 +154,7 @@ MultipleOutputs::Update()
 		return false;
 
 	for (auto ao : outputs)
-		ret = ao->LockUpdate(input_audio_format, *pipe)
+		ret = ao->LockUpdate(input_audio_format, *pipe, force)
 			|| ret;
 
 	return ret;
@@ -186,32 +167,27 @@ MultipleOutputs::SetReplayGainMode(ReplayGainMode mode)
 		ao->SetReplayGainMode(mode);
 }
 
-bool
-MultipleOutputs::Play(MusicChunk *chunk, Error &error)
+void
+MultipleOutputs::Play(MusicChunk *chunk)
 {
 	assert(buffer != nullptr);
 	assert(pipe != nullptr);
 	assert(chunk != nullptr);
 	assert(chunk->CheckFormat(input_audio_format));
 
-	if (!Update()) {
+	if (!Update(false))
 		/* TODO: obtain real error */
-		error.Set(output_domain, "Failed to open audio output");
-		return false;
-	}
+		throw std::runtime_error("Failed to open audio output");
 
 	pipe->Push(chunk);
 
 	for (auto ao : outputs)
 		ao->LockPlay();
-
-	return true;
 }
 
-bool
+void
 MultipleOutputs::Open(const AudioFormat audio_format,
-		      MusicBuffer &_buffer,
-		      Error &error)
+		      MusicBuffer &_buffer)
 {
 	bool ret = false, enabled = false;
 
@@ -233,71 +209,51 @@ MultipleOutputs::Open(const AudioFormat audio_format,
 
 	input_audio_format = audio_format;
 
-	ResetReopen();
 	EnableDisable();
-	Update();
+	Update(true);
+
+	std::exception_ptr first_error;
 
 	for (auto ao : outputs) {
-		if (ao->enabled)
+		const std::lock_guard<Mutex> lock(ao->mutex);
+
+		if (ao->IsEnabled())
 			enabled = true;
 
-		if (ao->open)
+		if (ao->IsOpen())
 			ret = true;
+		else if (!first_error)
+			first_error = ao->GetLastError();
 	}
 
-	if (!enabled)
-		error.Set(output_domain, "All audio outputs are disabled");
-	else if (!ret)
-		/* TODO: obtain real error */
-		error.Set(output_domain, "Failed to open audio output");
-
-	if (!ret)
+	if (!enabled) {
+		/* close all devices if there was an error */
+		Close();
+		throw std::runtime_error("All audio outputs are disabled");
+	} else if (!ret) {
 		/* close all devices if there was an error */
 		Close();
 
-	return ret;
-}
-
-/**
- * Has the specified audio output already consumed this chunk?
- */
-gcc_pure
-static bool
-chunk_is_consumed_in(const AudioOutput *ao,
-		     gcc_unused const MusicPipe *pipe,
-		     const MusicChunk *chunk)
-{
-	if (!ao->open)
-		return true;
-
-	if (ao->current_chunk == nullptr)
-		return false;
-
-	assert(chunk == ao->current_chunk ||
-	       pipe->Contains(ao->current_chunk));
-
-	if (chunk != ao->current_chunk) {
-		assert(chunk->next != nullptr);
-		return true;
+		if (first_error)
+			/* we have details, so throw that */
+			std::rethrow_exception(first_error);
+		else
+			throw std::runtime_error("Failed to open audio output");
 	}
-
-	return ao->current_chunk_finished && chunk->next == nullptr;
 }
 
 bool
-MultipleOutputs::IsChunkConsumed(const MusicChunk *chunk) const
+MultipleOutputs::IsChunkConsumed(const MusicChunk *chunk) const noexcept
 {
-	for (auto ao : outputs) {
-		const ScopeLock protect(ao->mutex);
-		if (!chunk_is_consumed_in(ao, pipe, chunk))
+	for (auto ao : outputs)
+		if (!ao->LockIsChunkConsumed(*chunk))
 			return false;
-	}
 
 	return true;
 }
 
 inline void
-MultipleOutputs::ClearTailChunk(gcc_unused const MusicChunk *chunk,
+MultipleOutputs::ClearTailChunk(const MusicChunk *chunk,
 				bool *locked)
 {
 	assert(chunk->next == nullptr);
@@ -309,16 +265,14 @@ MultipleOutputs::ClearTailChunk(gcc_unused const MusicChunk *chunk,
 		/* this mutex will be unlocked by the caller when it's
 		   ready */
 		ao->mutex.lock();
-		locked[i] = ao->open;
+		locked[i] = ao->IsOpen();
 
 		if (!locked[i]) {
 			ao->mutex.unlock();
 			continue;
 		}
 
-		assert(ao->current_chunk == chunk);
-		assert(ao->current_chunk_finished);
-		ao->current_chunk = nullptr;
+		ao->ClearTailChunk(*chunk);
 	}
 }
 
@@ -370,26 +324,10 @@ MultipleOutputs::Check()
 	return 0;
 }
 
-bool
-MultipleOutputs::Wait(PlayerControl &pc, unsigned threshold)
-{
-	pc.Lock();
-
-	if (Check() < threshold) {
-		pc.Unlock();
-		return true;
-	}
-
-	pc.Wait();
-	pc.Unlock();
-
-	return Check() < threshold;
-}
-
 void
 MultipleOutputs::Pause()
 {
-	Update();
+	Update(false);
 
 	for (auto ao : outputs)
 		ao->LockPauseAsync();
