@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * Copyright (C) 2010-2011 Philipp 'ph3-der-loewe' Schafft
  * Copyright (C) 2010-2011 Hans-Kristian 'maister' Arntzen
  *
@@ -21,13 +21,14 @@
 #include "config.h"
 #include "RoarOutputPlugin.hxx"
 #include "../OutputAPI.hxx"
+#include "../Wrapper.hxx"
 #include "mixer/MixerList.hxx"
 #include "thread/Mutex.hxx"
-#include "util/Error.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
 #include <string>
+#include <stdexcept>
 
 /* libroar/services.h declares roar_service_stream::new - work around
    this C++ problem */
@@ -36,50 +37,62 @@
 #undef new
 
 class RoarOutput {
+	friend struct AudioOutputWrapper<RoarOutput>;
+
 	AudioOutput base;
 
-	std::string host, name;
+	const std::string host, name;
 
 	roar_vs_t * vss;
-	int err;
-	int role;
+	int err = ROAR_ERROR_NONE;
+	const int role;
 	struct roar_connection con;
 	struct roar_audio_info info;
 	mutable Mutex mutex;
 	bool alive;
 
 public:
-	RoarOutput()
-		:base(roar_output_plugin),
-		 err(ROAR_ERROR_NONE) {}
+	RoarOutput(const ConfigBlock &block);
 
 	operator AudioOutput *() {
 		return &base;
 	}
 
-	bool Initialize(const config_param &param, Error &error) {
-		return base.Configure(param, error);
-	}
-
-	void Configure(const config_param &param);
-
-	bool Open(AudioFormat &audio_format, Error &error);
+	void Open(AudioFormat &audio_format);
 	void Close();
 
 	void SendTag(const Tag &tag);
-	size_t Play(const void *chunk, size_t size, Error &error);
+	size_t Play(const void *chunk, size_t size);
 	void Cancel();
 
 	int GetVolume() const;
-	bool SetVolume(unsigned volume);
+	void SetVolume(unsigned volume);
 };
 
 static constexpr Domain roar_output_domain("roar_output");
 
+gcc_pure
+static int
+GetConfiguredRole(const ConfigBlock &block) noexcept
+{
+	const char *role = block.GetBlockValue("role");
+	return role != nullptr
+		? roar_str2role(role)
+		: ROAR_ROLE_MUSIC;
+}
+
+RoarOutput::RoarOutput(const ConfigBlock &block)
+	:base(roar_output_plugin, block),
+	 host(block.GetBlockValue("server", "")),
+	 name(block.GetBlockValue("name", "MPD")),
+	 role(GetConfiguredRole(block))
+{
+}
+
 inline int
 RoarOutput::GetVolume() const
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	if (vss == nullptr || !alive)
 		return -1;
@@ -87,7 +100,7 @@ RoarOutput::GetVolume() const
 	float l, r;
 	int error;
 	if (roar_vs_volume_get(vss, &l, &r, &error) < 0)
-		return -1;
+		throw std::runtime_error(roar_vs_strerr(error));
 
 	return (l + r) * 50;
 }
@@ -98,60 +111,32 @@ roar_output_get_volume(RoarOutput &roar)
 	return roar.GetVolume();
 }
 
-bool
+inline void
 RoarOutput::SetVolume(unsigned volume)
 {
 	assert(volume <= 100);
 
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	if (vss == nullptr || !alive)
-		return false;
+		throw std::runtime_error("closed");
 
 	int error;
 	float level = volume / 100.0;
 
-	roar_vs_volume_mono(vss, level, &error);
-	return true;
+	if (roar_vs_volume_mono(vss, level, &error) < 0)
+		throw std::runtime_error(roar_vs_strerr(error));
 }
 
-bool
+void
 roar_output_set_volume(RoarOutput &roar, unsigned volume)
 {
-	return roar.SetVolume(volume);
-}
-
-inline void
-RoarOutput::Configure(const config_param &param)
-{
-	host = param.GetBlockValue("server", "");
-	name = param.GetBlockValue("name", "MPD");
-
-	const char *_role = param.GetBlockValue("role", "music");
-	role = _role != nullptr
-		? roar_str2role(_role)
-		: ROAR_ROLE_MUSIC;
+	roar.SetVolume(volume);
 }
 
 static AudioOutput *
-roar_init(const config_param &param, Error &error)
+roar_init(const ConfigBlock &block)
 {
-	RoarOutput *self = new RoarOutput();
-
-	if (!self->Initialize(param, error)) {
-		delete self;
-		return nullptr;
-	}
-
-	self->Configure(param);
-	return *self;
-}
-
-static void
-roar_finish(AudioOutput *ao)
-{
-	RoarOutput *self = (RoarOutput *)ao;
-
-	delete self;
+	return *new RoarOutput(block);
 }
 
 static void
@@ -189,50 +174,34 @@ roar_use_audio_format(struct roar_audio_info *info,
 	}
 }
 
-inline bool
-RoarOutput::Open(AudioFormat &audio_format, Error &error)
+inline void
+RoarOutput::Open(AudioFormat &audio_format)
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	if (roar_simple_connect(&con,
 				host.empty() ? nullptr : host.c_str(),
-				name.c_str()) < 0) {
-		error.Set(roar_output_domain,
-			  "Failed to connect to Roar server");
-		return false;
-	}
+				name.c_str()) < 0)
+		throw std::runtime_error("Failed to connect to Roar server");
 
 	vss = roar_vs_new_from_con(&con, &err);
 
-	if (vss == nullptr || err != ROAR_ERROR_NONE) {
-		error.Set(roar_output_domain, "Failed to connect to server");
-		return false;
-	}
+	if (vss == nullptr || err != ROAR_ERROR_NONE)
+		throw std::runtime_error("Failed to connect to server");
 
 	roar_use_audio_format(&info, audio_format);
 
-	if (roar_vs_stream(vss, &info, ROAR_DIR_PLAY, &err) < 0) {
-		error.Set(roar_output_domain, "Failed to start stream");
-		return false;
-	}
+	if (roar_vs_stream(vss, &info, ROAR_DIR_PLAY, &err) < 0)
+		throw std::runtime_error("Failed to start stream");
 
 	roar_vs_role(vss, role, &err);
 	alive = true;
-	return true;
-}
-
-static bool
-roar_open(AudioOutput *ao, AudioFormat &audio_format, Error &error)
-{
-	RoarOutput *self = (RoarOutput *)ao;
-
-	return self->Open(audio_format, error);
 }
 
 inline void
 RoarOutput::Close()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	alive = false;
 
@@ -242,17 +211,10 @@ RoarOutput::Close()
 	roar_disconnect(&con);
 }
 
-static void
-roar_close(AudioOutput *ao)
-{
-	RoarOutput *self = (RoarOutput *)ao;
-	self->Close();
-}
-
 inline void
 RoarOutput::Cancel()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	if (vss == nullptr)
 		return;
@@ -277,39 +239,19 @@ RoarOutput::Cancel()
 	alive = true;
 }
 
-static void
-roar_cancel(AudioOutput *ao)
-{
-	RoarOutput *self = (RoarOutput *)ao;
-
-	self->Cancel();
-}
-
 inline size_t
-RoarOutput::Play(const void *chunk, size_t size, Error &error)
+RoarOutput::Play(const void *chunk, size_t size)
 {
-	if (vss == nullptr) {
-		error.Set(roar_output_domain, "Connection is invalid");
-		return 0;
-	}
+	if (vss == nullptr)
+		throw std::runtime_error("Connection is invalid");
 
 	ssize_t nbytes = roar_vs_write(vss, chunk, size, &err);
-	if (nbytes <= 0) {
-		error.Set(roar_output_domain, "Failed to play data");
-		return 0;
-	}
+	if (nbytes <= 0)
+		throw std::runtime_error("Failed to play data");
 
 	return nbytes;
 }
 
-static size_t
-roar_play(AudioOutput *ao, const void *chunk, size_t size,
-	  Error &error)
-{
-	RoarOutput *self = (RoarOutput *)ao;
-	return self->Play(chunk, size, error);
-}
-
 static const char*
 roar_tag_convert(TagType type, bool *is_uuid)
 {
@@ -347,8 +289,6 @@ roar_tag_convert(TagType type, bool *is_uuid)
 		case TAG_MUSICBRAINZ_ALBUMID:
 		case TAG_MUSICBRAINZ_ALBUMARTISTID:
 		case TAG_MUSICBRAINZ_TRACKID:
-			*is_uuid = true;
-			return "HASH";
 		case TAG_MUSICBRAINZ_RELEASETRACKID:
 			*is_uuid = true;
 			return "HASH";
@@ -364,7 +304,7 @@ RoarOutput::SendTag(const Tag &tag)
 	if (vss == nullptr)
 		return;
 
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	size_t cnt = 0;
 	struct roar_keyval vals[32];
@@ -407,26 +347,28 @@ RoarOutput::SendTag(const Tag &tag)
 }
 
 static void
-roar_send_tag(AudioOutput *ao, const Tag *meta)
+roar_send_tag(AudioOutput *ao, const Tag &meta)
 {
 	RoarOutput *self = (RoarOutput *)ao;
-	self->SendTag(*meta);
+	self->SendTag(meta);
 }
 
+typedef AudioOutputWrapper<RoarOutput> Wrapper;
+
 const struct AudioOutputPlugin roar_output_plugin = {
 	"roar",
 	nullptr,
 	roar_init,
-	roar_finish,
+	&Wrapper::Finish,
 	nullptr,
 	nullptr,
-	roar_open,
-	roar_close,
+	&Wrapper::Open,
+	&Wrapper::Close,
 	nullptr,
 	roar_send_tag,
-	roar_play,
+	&Wrapper::Play,
 	nullptr,
-	roar_cancel,
+	&Wrapper::Cancel,
 	nullptr,
 	&roar_mixer_plugin,
 };
