@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,10 @@
 #include "HttpdClient.hxx"
 #include "HttpdInternal.hxx"
 #include "util/ASCII.hxx"
+#include "util/AllocatedString.hxx"
 #include "Page.hxx"
 #include "IcyMetaDataServer.hxx"
-#include "system/SocketError.hxx"
+#include "net/SocketError.hxx"
 #include "Log.hxx"
 
 #include <assert.h>
@@ -55,7 +56,7 @@ HttpdClient::Close()
 void
 HttpdClient::LockClose()
 {
-	const ScopeLock protect(httpd.mutex);
+	const std::lock_guard<Mutex> protect(httpd.mutex);
 	Close();
 }
 
@@ -141,7 +142,8 @@ HttpdClient::HandleLine(const char *line)
 bool
 HttpdClient::SendResponse()
 {
-	char buffer[1024], *allocated = nullptr;
+	char buffer[1024];
+	AllocatedString<> allocated = nullptr;
 	const char *response;
 
 	assert(state == RESPONSE);
@@ -162,11 +164,12 @@ HttpdClient::SendResponse()
 		response = buffer;
 
 	} else if (metadata_requested) {
-		response = allocated =
+		allocated =
 			icy_server_metadata_header(httpd.name, httpd.genre,
 						   httpd.website,
 						   httpd.content_type,
 						   metaint);
+		response = allocated.c_str();
        } else { /* revert to a normal HTTP request */
 		snprintf(buffer, sizeof(buffer),
 			 "HTTP/1.1 200 OK\r\n"
@@ -180,7 +183,6 @@ HttpdClient::SendResponse()
 	}
 
 	ssize_t nbytes = SocketMonitor::Write(response, strlen(response));
-	delete[] allocated;
 	if (gcc_unlikely(nbytes < 0)) {
 		const SocketErrorMessage msg;
 		FormatWarning(httpd_output_domain,
@@ -258,7 +260,7 @@ HttpdClient::TryWritePageN(const Page &page, size_t position, ssize_t n)
 }
 
 ssize_t
-HttpdClient::GetBytesTillMetaData() const
+HttpdClient::GetBytesTillMetaData() const noexcept
 {
 	if (metadata_requested &&
 	    current_page->size - current_position > metaint - metadata_fill)
@@ -270,7 +272,7 @@ HttpdClient::GetBytesTillMetaData() const
 inline bool
 HttpdClient::TryWrite()
 {
-	const ScopeLock protect(httpd.mutex);
+	const std::lock_guard<Mutex> protect(httpd.mutex);
 
 	assert(state == RESPONSE);
 
@@ -472,9 +474,9 @@ HttpdClient::OnSocketInput(void *data, size_t length)
 }
 
 void
-HttpdClient::OnSocketError(Error &&error)
+HttpdClient::OnSocketError(std::exception_ptr ep)
 {
-	LogError(error);
+	LogError(ep);
 }
 
 void
