@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 #include "Play.hxx"
 #include "AndroidSimpleBufferQueue.hxx"
 #include "../../OutputAPI.hxx"
+#include "../../Wrapper.hxx"
 #include "util/Macros.hxx"
-#include "util/Error.hxx"
 #include "util/Domain.hxx"
 #include "system/ByteOrder.hxx"
 #include "Log.hxx"
@@ -33,7 +33,11 @@
 #include <SLES/OpenSLES.h>
 #include <SLES/OpenSLES_Android.h>
 
+#include <stdexcept>
+
 class SlesOutput {
+	friend struct AudioOutputWrapper<SlesOutput>;
+
 	static constexpr unsigned N_BUFFERS = 3;
 	static constexpr size_t BUFFER_SIZE = 65536;
 
@@ -81,27 +85,24 @@ class SlesOutput {
 	uint8_t buffers[N_BUFFERS][BUFFER_SIZE];
 
 public:
-	SlesOutput()
-		:base(sles_output_plugin) {}
+	SlesOutput(const ConfigBlock &block);
 
 	operator AudioOutput *() {
 		return &base;
 	}
 
-	bool Initialize(const config_param &param, Error &error) {
-		return base.Configure(param, error);
-	}
+	static SlesOutput *Create(const ConfigBlock &block);
 
-	bool Configure(const config_param &param, Error &error);
-
-	bool Open(AudioFormat &audio_format, Error &error);
+	void Open(AudioFormat &audio_format);
 	void Close();
 
-	unsigned Delay() {
-		return pause && !cancel ? 100 : 0;
+	std::chrono::steady_clock::duration Delay() noexcept {
+		return pause && !cancel
+			? std::chrono::milliseconds(100)
+			: std::chrono::steady_clock::duration::zero();
 	}
 
-	size_t Play(const void *chunk, size_t size, Error &error);
+	size_t Play(const void *chunk, size_t size);
 
 	void Drain();
 	void Cancel();
@@ -125,62 +126,52 @@ private:
 
 static constexpr Domain sles_domain("sles");
 
-inline bool
-SlesOutput::Configure(const config_param &, Error &)
+SlesOutput::SlesOutput(const ConfigBlock &block)
+	:base(sles_output_plugin, block)
 {
-	return true;
 }
 
-inline bool
-SlesOutput::Open(AudioFormat &audio_format, Error &error)
+inline void
+SlesOutput::Open(AudioFormat &audio_format)
 {
 	SLresult result;
 	SLObjectItf _object;
 
 	result = slCreateEngine(&_object, 0, nullptr, 0,
 				nullptr, nullptr);
-	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result), "slCreateEngine() failed");
-		return false;
-	}
+	if (result != SL_RESULT_SUCCESS)
+		throw std::runtime_error("slCreateEngine() failed");
 
 	engine_object = SLES::Object(_object);
 
 	result = engine_object.Realize(false);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result), "Engine.Realize() failed");
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Engine.Realize() failed");
 	}
 
 	SLEngineItf _engine;
 	result = engine_object.GetInterface(SL_IID_ENGINE, &_engine);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Engine.GetInterface(IID_ENGINE) failed");
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Engine.GetInterface(IID_ENGINE) failed");
 	}
 
 	SLES::Engine engine(_engine);
 
 	result = engine.CreateOutputMix(&_object, 0, nullptr, nullptr);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Engine.CreateOutputMix() failed");
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Engine.CreateOutputMix() failed");
 	}
 
 	mix_object = SLES::Object(_object);
 
 	result = mix_object.Realize(false);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Mix.Realize() failed");
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Mix.Realize() failed");
 	}
 
 	SLDataLocator_AndroidSimpleBufferQueue loc_bufq = {
@@ -233,11 +224,9 @@ SlesOutput::Open(AudioFormat &audio_format, Error &error)
 	result = engine.CreateAudioPlayer(&_object, &audioSrc, &audioSnk,
 					  ARRAY_SIZE(ids2), ids2, req2);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Engine.CreateAudioPlayer() failed");
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Engine.CreateAudioPlayer() failed");
 	}
 
 	play_object = SLES::Object(_object);
@@ -255,23 +244,19 @@ SlesOutput::Open(AudioFormat &audio_format, Error &error)
 	result = play_object.Realize(false);
 
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Play.Realize() failed");
 		play_object.Destroy();
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Play.Realize() failed");
 	}
 
 	SLPlayItf _play;
 	result = play_object.GetInterface(SL_IID_PLAY, &_play);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Play.GetInterface(IID_PLAY) failed");
 		play_object.Destroy();
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Play.GetInterface(IID_PLAY) failed");
 	}
 
 	play = SLES::Play(_play);
@@ -280,33 +265,27 @@ SlesOutput::Open(AudioFormat &audio_format, Error &error)
 	result = play_object.GetInterface(SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
 					  &_queue);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Play.GetInterface(IID_ANDROIDSIMPLEBUFFERQUEUE) failed");
 		play_object.Destroy();
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Play.GetInterface(IID_ANDROIDSIMPLEBUFFERQUEUE) failed");
 	}
 
 	queue = SLES::AndroidSimpleBufferQueue(_queue);
 	result = queue.RegisterCallback(PlayedCallback, (void *)this);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Play.RegisterCallback() failed");
 		play_object.Destroy();
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Play.RegisterCallback() failed");
 	}
 
 	result = play.SetPlayState(SL_PLAYSTATE_PLAYING);
 	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "Play.SetPlayState(PLAYING) failed");
 		play_object.Destroy();
 		mix_object.Destroy();
 		engine_object.Destroy();
-		return false;
+		throw std::runtime_error("Play.SetPlayState(PLAYING) failed");
 	}
 
 	pause = cancel = false;
@@ -316,8 +295,6 @@ SlesOutput::Open(AudioFormat &audio_format, Error &error)
 
 	// TODO: support other sample formats
 	audio_format.format = SampleFormat::S16;
-
-	return true;
 }
 
 inline void
@@ -330,22 +307,19 @@ SlesOutput::Close()
 }
 
 inline size_t
-SlesOutput::Play(const void *chunk, size_t size, Error &error)
+SlesOutput::Play(const void *chunk, size_t size)
 {
 	cancel = false;
 
 	if (pause) {
 		SLresult result = play.SetPlayState(SL_PLAYSTATE_PLAYING);
-		if (result != SL_RESULT_SUCCESS) {
-			error.Set(sles_domain, int(result),
-				  "Play.SetPlayState(PLAYING) failed");
-			return false;
-		}
+		if (result != SL_RESULT_SUCCESS)
+			throw std::runtime_error("Play.SetPlayState(PLAYING) failed");
 
 		pause = false;
 	}
 
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	assert(filled < BUFFER_SIZE);
 
@@ -361,11 +335,8 @@ SlesOutput::Play(const void *chunk, size_t size, Error &error)
 		return nbytes;
 
 	SLresult result = queue.Enqueue(buffers[next], BUFFER_SIZE);
-	if (result != SL_RESULT_SUCCESS) {
-		error.Set(sles_domain, int(result),
-			  "AndroidSimpleBufferQueue.Enqueue() failed");
-		return 0;
-	}
+	if (result != SL_RESULT_SUCCESS)
+		throw std::runtime_error("AndroidSimpleBufferQueue.Enqueue() failed");
 
 	++n_queued;
 	next = (next + 1) % N_BUFFERS;
@@ -377,7 +348,7 @@ SlesOutput::Play(const void *chunk, size_t size, Error &error)
 inline void
 SlesOutput::Drain()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 
 	assert(filled < BUFFER_SIZE);
 
@@ -400,7 +371,7 @@ SlesOutput::Cancel()
 		FormatWarning(sles_domain,
 			      "AndroidSimpleBufferQueue.Clear() failed");
 
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	n_queued = 0;
 	filled = 0;
 }
@@ -427,7 +398,7 @@ SlesOutput::Pause()
 inline void
 SlesOutput::PlayedCallback()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	assert(n_queued > 0);
 	--n_queued;
 	cond.signal();
@@ -441,99 +412,28 @@ sles_test_default_device()
 	return true;
 }
 
-static AudioOutput *
-sles_output_init(const config_param &param, Error &error)
+inline SlesOutput *
+SlesOutput::Create(const ConfigBlock &block)
 {
-	SlesOutput *sles = new SlesOutput();
-
-	if (!sles->Initialize(param, error) ||
-	    !sles->Configure(param, error)) {
-		delete sles;
-		return nullptr;
-	}
-
-	return *sles;
+	return new SlesOutput(block);
 }
 
-static void
-sles_output_finish(AudioOutput *ao)
-{
-	SlesOutput *sles = (SlesOutput *)ao;
-
-	delete sles;
-}
-
-static bool
-sles_output_open(AudioOutput *ao, AudioFormat &audio_format, Error &error)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	return sles.Open(audio_format, error);
-}
-
-static void
-sles_output_close(AudioOutput *ao)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	sles.Close();
-}
-
-static unsigned
-sles_output_delay(AudioOutput *ao)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	return sles.Delay();
-}
-
-static size_t
-sles_output_play(AudioOutput *ao, const void *chunk, size_t size,
-		 Error &error)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	return sles.Play(chunk, size, error);
-}
-
-static void
-sles_output_drain(AudioOutput *ao)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	sles.Drain();
-}
-
-static void
-sles_output_cancel(AudioOutput *ao)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	sles.Cancel();
-}
-
-static bool
-sles_output_pause(AudioOutput *ao)
-{
-	SlesOutput &sles = *(SlesOutput *)ao;
-
-	return sles.Pause();
-}
+typedef AudioOutputWrapper<SlesOutput> Wrapper;
 
 const struct AudioOutputPlugin sles_output_plugin = {
 	"sles",
 	sles_test_default_device,
-	sles_output_init,
-	sles_output_finish,
+	&Wrapper::Init,
+	&Wrapper::Finish,
 	nullptr,
 	nullptr,
-	sles_output_open,
-	sles_output_close,
-	sles_output_delay,
+	&Wrapper::Open,
+	&Wrapper::Close,
+	&Wrapper::Delay,
 	nullptr,
-	sles_output_play,
-	sles_output_drain,
-	sles_output_cancel,
-	sles_output_pause,
+	&Wrapper::Play,
+	&Wrapper::Drain,
+	&Wrapper::Cancel,
+	&Wrapper::Pause,
 	nullptr,
 };
