@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2016 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,8 @@
 
 #include "config.h"
 #include "Silence.hxx"
-#include "AudioFormat.hxx"
+#include "Traits.hxx"
+#include "SampleFormat.hxx"
 #include "util/WritableBuffer.hxx"
 
 #include <string.h>
@@ -29,7 +30,7 @@ PcmSilence(WritableBuffer<void> dest, SampleFormat format)
 {
 	uint8_t pattern = 0;
 	if (format == SampleFormat::DSD)
-		pattern = 0x69;
+		pattern = SampleTraits<SampleFormat::DSD>::SILENCE;
 
 	memset(dest.data, pattern, dest.size);
 }
