@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,10 +24,10 @@
 #include "tag/TagBuilder.hxx"
 #include "fs/Traits.hxx"
 #include "util/UriUtil.hxx"
-#include "util/Error.hxx"
 #include "DetachedSong.hxx"
 
-#include <assert.h>
+#include <stdexcept>
+
 #include <string.h>
 
 static void
@@ -45,7 +45,14 @@ merge_song_metadata(DetachedSong &add, const DetachedSong &base)
 static bool
 playlist_check_load_song(DetachedSong &song, const SongLoader &loader)
 {
-	DetachedSong *tmp = loader.LoadSong(song.GetURI(), IgnoreError());
+	DetachedSong *tmp;
+
+	try {
+		tmp = loader.LoadSong(song.GetURI());
+	} catch (const std::runtime_error &) {
+		return false;
+	}
+
 	if (tmp == nullptr)
 		return false;
 
