@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,12 +23,12 @@
 #include "../MemorySongEnumerator.hxx"
 #include "tag/TagBuilder.hxx"
 #include "util/ASCII.hxx"
-#include "util/Error.hxx"
+#include "util/StringView.hxx"
 #include "lib/expat/ExpatParser.hxx"
 #include "Log.hxx"
 
 /**
- * This is the state object for the GLib XML parser.
+ * This is the state object for our XML parser.
  */
 struct AsxParser {
 	/**
@@ -130,7 +130,8 @@ asx_char_data(void *user_data, const XML_Char *s, int len)
 
 	case AsxParser::ENTRY:
 		if (parser->tag_type != TAG_NUM_OF_ITEM_TYPES)
-			parser->tag_builder.AddItem(parser->tag_type, s, len);
+			parser->tag_builder.AddItem(parser->tag_type,
+						    StringView(s, len));
 
 		break;
 	}
@@ -142,7 +143,7 @@ asx_char_data(void *user_data, const XML_Char *s, int len)
  */
 
 static SongEnumerator *
-asx_open_stream(InputStream &is)
+asx_open_stream(InputStreamPtr &&is)
 {
 	AsxParser parser;
 
@@ -150,12 +151,7 @@ asx_open_stream(InputStream &is)
 		ExpatParser expat(&parser);
 		expat.SetElementHandler(asx_start_element, asx_end_element);
 		expat.SetCharacterDataHandler(asx_char_data);
-
-		Error error;
-		if (!expat.Parse(is, error)) {
-			LogError(error);
-			return nullptr;
-		}
+		expat.Parse(*is);
 	}
 
 	parser.songs.reverse();
