@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,7 @@
 #include "../SongEnumerator.hxx"
 #include "../cue/CueParser.hxx"
 #include "tag/TagHandler.hxx"
-#include "tag/TagId3.hxx"
-#include "tag/ApeTag.hxx"
+#include "tag/Generic.hxx"
 #include "DetachedSong.hxx"
 #include "TagFile.hxx"
 #include "fs/Traits.hxx"
@@ -69,7 +68,7 @@ public:
 		delete parser;
 	}
 
-	virtual DetachedSong *NextSong() override;
+	virtual std::unique_ptr<DetachedSong> NextSong() override;
 };
 
 static void
@@ -82,7 +81,7 @@ embcue_tag_pair(const char *name, const char *value, void *ctx)
 		playlist->cuesheet = value;
 }
 
-static const struct tag_handler embcue_tag_handler = {
+static constexpr TagHandler embcue_tag_handler = {
 	nullptr,
 	nullptr,
 	embcue_tag_pair,
@@ -97,18 +96,13 @@ embcue_playlist_open_uri(const char *uri,
 		/* only local files supported */
 		return nullptr;
 
-	const auto path_fs = AllocatedPath::FromUTF8(uri);
-	if (path_fs.IsNull())
-		return nullptr;
+	const auto path_fs = AllocatedPath::FromUTF8Throw(uri);
 
 	const auto playlist = new EmbeddedCuePlaylist();
 
 	tag_file_scan(path_fs, embcue_tag_handler, playlist);
-	if (playlist->cuesheet.empty()) {
-		tag_ape_scan2(path_fs, &embcue_tag_handler, playlist);
-		if (playlist->cuesheet.empty())
-			tag_id3_scan(path_fs, &embcue_tag_handler, playlist);
-	}
+	if (playlist->cuesheet.empty())
+		ScanGenericTags(path_fs, embcue_tag_handler, playlist);
 
 	if (playlist->cuesheet.empty()) {
 		/* no "CUESHEET" tag found */
@@ -124,12 +118,14 @@ embcue_playlist_open_uri(const char *uri,
 	return playlist;
 }
 
-DetachedSong *
+std::unique_ptr<DetachedSong>
 EmbeddedCuePlaylist::NextSong()
 {
-	DetachedSong *song = parser->Get();
-	if (song != nullptr)
+	auto song = parser->Get();
+	if (song != nullptr) {
+		song->SetURI(filename);
 		return song;
+	}
 
 	while (*next != 0) {
 		const char *line = next;
