@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,14 +24,14 @@
 #include "DetachedSong.hxx"
 #include "input/InputStream.hxx"
 #include "tag/TagBuilder.hxx"
-#include "util/Error.hxx"
+#include "util/StringView.hxx"
 #include "lib/expat/ExpatParser.hxx"
 #include "Log.hxx"
 
 #include <string.h>
 
 /**
- * This is the state object for the GLib XML parser.
+ * This is the state object for our XML parser.
  */
 struct XspfParser {
 	/**
@@ -170,7 +170,8 @@ xspf_char_data(void *user_data, const XML_Char *s, int len)
 	case XspfParser::TRACK:
 		if (!parser->location.empty() &&
 		    parser->tag_type != TAG_NUM_OF_ITEM_TYPES)
-			parser->tag_builder.AddItem(parser->tag_type, s, len);
+			parser->tag_builder.AddItem(parser->tag_type,
+						    StringView(s, len));
 
 		break;
 
@@ -187,7 +188,7 @@ xspf_char_data(void *user_data, const XML_Char *s, int len)
  */
 
 static SongEnumerator *
-xspf_open_stream(InputStream &is)
+xspf_open_stream(InputStreamPtr &&is)
 {
 	XspfParser parser;
 
@@ -195,12 +196,7 @@ xspf_open_stream(InputStream &is)
 		ExpatParser expat(&parser);
 		expat.SetElementHandler(xspf_start_element, xspf_end_element);
 		expat.SetCharacterDataHandler(xspf_char_data);
-
-		Error error;
-		if (!expat.Parse(is, error)) {
-			LogError(error);
-			return nullptr;
-		}
+		expat.Parse(*is);
 	}
 
 	parser.songs.reverse();
