@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,13 +36,11 @@ MemoryStorageDirectoryReader::Read()
 	return entries.front().name.c_str();
 }
 
-bool
-MemoryStorageDirectoryReader::GetInfo(gcc_unused bool follow, FileInfo &info,
-				      gcc_unused Error &error)
+StorageFileInfo
+MemoryStorageDirectoryReader::GetInfo(gcc_unused bool follow)
 {
 	assert(!first);
 	assert(!entries.empty());
 
-	info = entries.front().info;
-	return true;
+	return entries.front().info;
 }
