@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,19 +24,18 @@
 #include "storage/FileInfo.hxx"
 #include "storage/MemoryDirectoryReader.hxx"
 #include "lib/nfs/Blocking.hxx"
-#include "lib/nfs/Domain.hxx"
 #include "lib/nfs/Base.hxx"
 #include "lib/nfs/Lease.hxx"
 #include "lib/nfs/Connection.hxx"
 #include "lib/nfs/Glue.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "util/Error.hxx"
 #include "thread/Mutex.hxx"
 #include "thread/Cond.hxx"
 #include "event/Loop.hxx"
 #include "event/Call.hxx"
 #include "event/DeferredMonitor.hxx"
 #include "event/TimeoutMonitor.hxx"
+#include "util/StringCompare.hxx"
 
 extern "C" {
 #include <nfsc/libnfs.h>
@@ -65,7 +64,7 @@ class NfsStorage final
 	Mutex mutex;
 	Cond cond;
 	State state;
-	Error last_error;
+	std::exception_ptr last_exception;
 
 public:
 	NfsStorage(EventLoop &_loop, const char *_base,
@@ -84,15 +83,13 @@ public:
 	}
 
 	/* virtual methods from class Storage */
-	bool GetInfo(const char *uri_utf8, bool follow, FileInfo &info,
-		     Error &error) override;
+	StorageFileInfo GetInfo(const char *uri_utf8, bool follow) override;
 
-	StorageDirectoryReader *OpenDirectory(const char *uri_utf8,
-					      Error &error) override;
+	StorageDirectoryReader *OpenDirectory(const char *uri_utf8) override;
 
-	std::string MapUTF8(const char *uri_utf8) const override;
+	std::string MapUTF8(const char *uri_utf8) const noexcept override;
 
-	const char *MapToRelativeUTF8(const char *uri_utf8) const override;
+	const char *MapToRelativeUTF8(const char *uri_utf8) const noexcept override;
 
 	/* virtual methods from NfsLease */
 	void OnNfsConnectionReady() final {
@@ -101,18 +98,18 @@ public:
 		SetState(State::READY);
 	}
 
-	void OnNfsConnectionFailed(gcc_unused const Error &error) final {
+	void OnNfsConnectionFailed(std::exception_ptr e) final {
 		assert(state == State::CONNECTING);
 
-		SetState(State::DELAY, error);
-		TimeoutMonitor::ScheduleSeconds(60);
+		SetState(State::DELAY, std::move(e));
+		TimeoutMonitor::Schedule(std::chrono::minutes(1));
 	}
 
-	void OnNfsConnectionDisconnected(gcc_unused const Error &error) final {
+	void OnNfsConnectionDisconnected(std::exception_ptr e) final {
 		assert(state == State::READY);
 
-		SetState(State::DELAY, error);
-		TimeoutMonitor::ScheduleSeconds(5);
+		SetState(State::DELAY, std::move(e));
+		TimeoutMonitor::Schedule(std::chrono::seconds(5));
 	}
 
 	/* virtual methods from DeferredMonitor */
@@ -136,18 +133,17 @@ private:
 	void SetState(State _state) {
 		assert(GetEventLoop().IsInside());
 
-		const ScopeLock protect(mutex);
+		const std::lock_guard<Mutex> protect(mutex);
 		state = _state;
 		cond.broadcast();
 	}
 
-	void SetState(State _state, const Error &error) {
+	void SetState(State _state, std::exception_ptr &&e) {
 		assert(GetEventLoop().IsInside());
 
-		const ScopeLock protect(mutex);
+		const std::lock_guard<Mutex> protect(mutex);
 		state = _state;
-		last_error.Clear();
-		last_error.Set(error);
+		last_exception = std::move(e);
 		cond.broadcast();
 	}
 
@@ -167,8 +163,8 @@ private:
 			Connect();
 	}
 
-	bool WaitConnected(Error &error) {
-		const ScopeLock protect(mutex);
+	void WaitConnected() {
+		const std::lock_guard<Mutex> protect(mutex);
 
 		while (true) {
 			switch (state) {
@@ -183,12 +179,11 @@ private:
 
 			case State::CONNECTING:
 			case State::READY:
-				return true;
+				return;
 
 			case State::DELAY:
-				assert(last_error.IsDefined());
-				error.Set(last_error);
-				return false;
+				assert(last_exception);
+				std::rethrow_exception(last_exception);
 			}
 		}
 	}
@@ -216,7 +211,7 @@ private:
 };
 
 static std::string
-UriToNfsPath(const char *_uri_utf8, Error &error)
+UriToNfsPath(const char *_uri_utf8)
 {
 	assert(_uri_utf8 != nullptr);
 
@@ -224,35 +219,40 @@ UriToNfsPath(const char *_uri_utf8, Error &error)
 	std::string uri_utf8("/");
 	uri_utf8.append(_uri_utf8);
 
-	return AllocatedPath::FromUTF8(uri_utf8.c_str(), error).Steal();
+#ifdef _WIN32
+	/* assume UTF-8 when accessing NFS from Windows */
+	return uri_utf8;
+#else
+	return AllocatedPath::FromUTF8Throw(uri_utf8.c_str()).Steal();
+#endif
 }
 
 std::string
-NfsStorage::MapUTF8(const char *uri_utf8) const
+NfsStorage::MapUTF8(const char *uri_utf8) const noexcept
 {
 	assert(uri_utf8 != nullptr);
 
-	if (*uri_utf8 == 0)
+	if (StringIsEmpty(uri_utf8))
 		return base;
 
 	return PathTraitsUTF8::Build(base.c_str(), uri_utf8);
 }
 
 const char *
-NfsStorage::MapToRelativeUTF8(const char *uri_utf8) const
+NfsStorage::MapToRelativeUTF8(const char *uri_utf8) const noexcept
 {
 	return PathTraitsUTF8::Relative(base.c_str(), uri_utf8);
 }
 
 static void
-Copy(FileInfo &info, const struct stat &st)
+Copy(StorageFileInfo &info, const struct stat &st)
 {
 	if (S_ISREG(st.st_mode))
-		info.type = FileInfo::Type::REGULAR;
+		info.type = StorageFileInfo::Type::REGULAR;
 	else if (S_ISDIR(st.st_mode))
-		info.type = FileInfo::Type::DIRECTORY;
+		info.type = StorageFileInfo::Type::DIRECTORY;
 	else
-		info.type = FileInfo::Type::OTHER;
+		info.type = StorageFileInfo::Type::OTHER;
 
 	info.size = st.st_size;
 	info.mtime = st.st_mtime;
@@ -262,16 +262,19 @@ Copy(FileInfo &info, const struct stat &st)
 
 class NfsGetInfoOperation final : public BlockingNfsOperation {
 	const char *const path;
-	FileInfo &info;
+	StorageFileInfo info;
 
 public:
-	NfsGetInfoOperation(NfsConnection &_connection, const char *_path,
-			    FileInfo &_info)
-		:BlockingNfsOperation(_connection), path(_path), info(_info) {}
+	NfsGetInfoOperation(NfsConnection &_connection, const char *_path)
+		:BlockingNfsOperation(_connection), path(_path) {}
+
+	const StorageFileInfo &GetInfo() const {
+		return info;
+	}
 
 protected:
-	bool Start(Error &_error) override {
-		return connection.Stat(path, *this, _error);
+	void Start() override {
+		connection.Stat(path, *this);
 	}
 
 	void HandleResult(gcc_unused unsigned status, void *data) override {
@@ -279,24 +282,21 @@ protected:
 	}
 };
 
-bool
-NfsStorage::GetInfo(const char *uri_utf8, gcc_unused bool follow,
-		    FileInfo &info, Error &error)
+StorageFileInfo
+NfsStorage::GetInfo(const char *uri_utf8, gcc_unused bool follow)
 {
-	const std::string path = UriToNfsPath(uri_utf8, error);
-	if (path.empty())
-		return false;
+	const std::string path = UriToNfsPath(uri_utf8);
 
-	if (!WaitConnected(error))
-		return false;
+	WaitConnected();
 
-	NfsGetInfoOperation operation(*connection, path.c_str(), info);
-	return operation.Run(error);
+	NfsGetInfoOperation operation(*connection, path.c_str());
+	operation.Run();
+	return operation.GetInfo();
 }
 
 gcc_pure
 static bool
-SkipNameFS(const char *name)
+SkipNameFS(PathTraitsFS::const_pointer_type name) noexcept
 {
 	return name[0] == '.' &&
 		(name[1] == 0 ||
@@ -304,19 +304,19 @@ SkipNameFS(const char *name)
 }
 
 static void
-Copy(FileInfo &info, const struct nfsdirent &ent)
+Copy(StorageFileInfo &info, const struct nfsdirent &ent)
 {
 	switch (ent.type) {
 	case NF3REG:
-		info.type = FileInfo::Type::REGULAR;
+		info.type = StorageFileInfo::Type::REGULAR;
 		break;
 
 	case NF3DIR:
-		info.type = FileInfo::Type::DIRECTORY;
+		info.type = StorageFileInfo::Type::DIRECTORY;
 		break;
 
 	default:
-		info.type = FileInfo::Type::OTHER;
+		info.type = StorageFileInfo::Type::OTHER;
 		break;
 	}
 
@@ -341,8 +341,8 @@ public:
 	}
 
 protected:
-	bool Start(Error &_error) override {
-		return connection.OpenDirectory(path, *this, _error);
+	void Start() override {
+		connection.OpenDirectory(path, *this);
 	}
 
 	void HandleResult(gcc_unused unsigned status, void *data) override {
@@ -363,7 +363,14 @@ NfsListDirectoryOperation::CollectEntries(struct nfsdir *dir)
 
 	const struct nfsdirent *ent;
 	while ((ent = connection.ReadDirectory(dir)) != nullptr) {
+#ifdef _WIN32
+		/* assume UTF-8 when accessing NFS from Windows */
+		const auto name_fs = AllocatedPath::FromUTF8Throw(ent->name);
+		if (name_fs.IsNull())
+			continue;
+#else
 		const Path name_fs = Path::FromFS(ent->name);
+#endif
 		if (SkipNameFS(name_fs.c_str()))
 			continue;
 
@@ -379,36 +386,29 @@ NfsListDirectoryOperation::CollectEntries(struct nfsdir *dir)
 }
 
 StorageDirectoryReader *
-NfsStorage::OpenDirectory(const char *uri_utf8, Error &error)
+NfsStorage::OpenDirectory(const char *uri_utf8)
 {
-	const std::string path = UriToNfsPath(uri_utf8, error);
-	if (path.empty())
-		return nullptr;
+	const std::string path = UriToNfsPath(uri_utf8);
 
-	if (!WaitConnected(error))
-		return nullptr;
+	WaitConnected();
 
 	NfsListDirectoryOperation operation(*connection, path.c_str());
-	if (!operation.Run(error))
-		return nullptr;
+	operation.Run();
 
 	return operation.ToReader();
 }
 
 static Storage *
-CreateNfsStorageURI(EventLoop &event_loop, const char *base,
-		    Error &error)
+CreateNfsStorageURI(EventLoop &event_loop, const char *base)
 {
-	if (memcmp(base, "nfs://", 6) != 0)
+	if (strncmp(base, "nfs://", 6) != 0)
 		return nullptr;
 
 	const char *p = base + 6;
 
 	const char *mount = strchr(p, '/');
-	if (mount == nullptr) {
-		error.Set(nfs_domain, "Malformed nfs:// URI");
-		return nullptr;
-	}
+	if (mount == nullptr)
+		throw std::runtime_error("Malformed nfs:// URI");
 
 	const std::string server(p, mount);
 
