@@ -19,7 +19,7 @@
 
 #include "Clock.hxx"
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 
 gcc_const
