@@ -49,7 +49,7 @@ aiff_seek_id3(InputStream &is)
 	aiff_header header;
 	is.ReadFull(&header, sizeof(header));
 	if (memcmp(header.id, "FORM", 4) != 0 ||
-	    (is.KnownSize() && FromLE32(header.size) > is.GetSize()) ||
+	    (is.KnownSize() && FromBE32(header.size) > is.GetSize()) ||
 	    (memcmp(header.format, "AIFF", 4) != 0 &&
 	     memcmp(header.format, "AIFC", 4) != 0))
 		throw std::runtime_error("Not an AIFF file");
