@@ -53,7 +53,7 @@ ParseMixRampTag(MixRampInfo &info, const char *name, const char *value)
 		const char *value;
 
 		gcc_pure
-		const char *operator[](const char *n) const {
+		const char *operator[](const char *n) const noexcept {
 			return StringEqualsCaseASCII(name, n)
 				? value
 				: nullptr;
@@ -70,7 +70,7 @@ ParseMixRampVorbis(MixRampInfo &info, const char *entry)
 		const char *entry;
 
 		gcc_pure
-		const char *operator[](const char *n) const {
+		const char *operator[](const char *n) const noexcept {
 			return vorbis_comment_value(entry, n);
 		}
 	};
