@@ -40,8 +40,10 @@
 
 #ifdef __linux__
 
+#ifndef ANDROID
+
 static int
-ioprio_set(int which, int who, int ioprio)
+linux_ioprio_set(int which, int who, int ioprio)
 {
 	return syscall(__NR_ioprio_set, which, who, ioprio);
 }
@@ -55,7 +57,20 @@ ioprio_set_idle()
 	static constexpr int _IOPRIO_IDLE =
 		(_IOPRIO_CLASS_IDLE << _IOPRIO_CLASS_SHIFT) | 7;
 
-	ioprio_set(_IOPRIO_WHO_PROCESS, 0, _IOPRIO_IDLE);
+	linux_ioprio_set(_IOPRIO_WHO_PROCESS, 0, _IOPRIO_IDLE);
+}
+
+#endif /* !ANDROID */
+
+/**
+ * Wrapper for the "sched_setscheduler" system call.  We don't use the
+ * one from the C library because Musl has an intentionally broken
+ * implementation.
+ */
+static int
+linux_sched_setscheduler(pid_t pid, int sched, const struct sched_param *param)
+{
+	return syscall(__NR_sched_setscheduler, pid, sched, param);
 }
 
 #endif
@@ -66,10 +81,14 @@ SetThreadIdlePriority()
 #ifdef __linux__
 #ifdef SCHED_IDLE
 	static struct sched_param sched_param;
-	sched_setscheduler(0, SCHED_IDLE, &sched_param);
+	linux_sched_setscheduler(0, SCHED_IDLE, &sched_param);
 #endif
 
+#ifndef ANDROID
+	/* this system call is forbidden via seccomp on Android 8 and
+	   leads to crash (SIGSYS) */
 	ioprio_set_idle();
+#endif
 
 #elif defined(_WIN32)
 	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_IDLE);
@@ -88,7 +107,7 @@ SetThreadRealtime()
 	policy |= SCHED_RESET_ON_FORK;
 #endif
 
-	if (sched_setscheduler(0, policy, &sched_param) < 0)
+	if (linux_sched_setscheduler(0, policy, &sched_param) < 0)
 		throw MakeErrno("sched_setscheduler failed");
 #endif	// __linux__
 };
