@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,17 +18,66 @@
  */
 
 #include "UriUtil.hxx"
+#include "StringCompare.hxx"
+#include "CharUtil.hxx"
 
 #include <assert.h>
 #include <string.h>
 
-bool uri_has_scheme(const char *uri)
+static constexpr bool
+IsValidSchemeStart(char ch)
+{
+	return IsLowerAlphaASCII(ch);
+}
+
+static constexpr bool
+IsValidSchemeChar(char ch)
+{
+	return IsLowerAlphaASCII(ch) || IsDigitASCII(ch) ||
+		ch == '+' || ch == '.' || ch == '-';
+}
+
+gcc_pure
+static bool
+IsValidScheme(StringView p) noexcept
+{
+	if (p.IsEmpty() || !IsValidSchemeStart(p.front()))
+		return false;
+
+	for (size_t i = 1; i < p.size; ++i)
+		if (!IsValidSchemeChar(p[i]))
+			return false;
+
+	return true;
+}
+
+/**
+ * Return the URI part after the scheme specification (and after the
+ * double slash).
+ */
+gcc_pure
+static const char *
+uri_after_scheme(const char *uri) noexcept
+{
+	if (uri[0] == '/' && uri[1] == '/' && uri[2] != '/')
+		return uri + 2;
+
+	const char *colon = strchr(uri, ':');
+	return colon != nullptr &&
+		IsValidScheme({uri, colon}) &&
+		colon[1] == '/' && colon[2] == '/'
+		? colon + 3
+		: nullptr;
+}
+
+bool
+uri_has_scheme(const char *uri) noexcept
 {
 	return strstr(uri, "://") != nullptr;
 }
 
 std::string
-uri_get_scheme(const char *uri)
+uri_get_scheme(const char *uri) noexcept
 {
 	const char *end = strstr(uri, "://");
 	if (end == nullptr)
@@ -37,9 +86,19 @@ uri_get_scheme(const char *uri)
 	return std::string(uri, end);
 }
 
+const char *
+uri_get_path(const char *uri) noexcept
+{
+	const char *ap = uri_after_scheme(uri);
+	if (ap != nullptr)
+		return strchr(ap, '/');
+
+	return uri;
+}
+
 /* suffixes should be ascii only characters */
 const char *
-uri_get_suffix(const char *uri)
+uri_get_suffix(const char *uri) noexcept
 {
 	const char *suffix = strrchr(uri, '.');
 	if (suffix == nullptr || suffix == uri ||
@@ -55,7 +114,7 @@ uri_get_suffix(const char *uri)
 }
 
 const char *
-uri_get_suffix(const char *uri, UriSuffixBuffer &buffer)
+uri_get_suffix(const char *uri, UriSuffixBuffer &buffer) noexcept
 {
 	const char *suffix = uri_get_suffix(uri);
 	if (suffix == nullptr)
@@ -72,10 +131,8 @@ uri_get_suffix(const char *uri, UriSuffixBuffer &buffer)
 }
 
 static const char *
-verify_uri_segment(const char *p)
+verify_uri_segment(const char *p) noexcept
 {
-	const char *q;
-
 	unsigned dots = 0;
 	while (*p == '.') {
 		++p;
@@ -85,12 +142,12 @@ verify_uri_segment(const char *p)
 	if (dots <= 2 && (*p == 0 || *p == '/'))
 		return nullptr;
 
-	q = strchr(p + 1, '/');
+	const char *q = strchr(p + 1, '/');
 	return q != nullptr ? q : "";
 }
 
 bool
-uri_safe_local(const char *uri)
+uri_safe_local(const char *uri) noexcept
 {
 	while (true) {
 		uri = verify_uri_segment(uri);
@@ -106,26 +163,33 @@ uri_safe_local(const char *uri)
 	}
 }
 
-std::string
-uri_remove_auth(const char *uri)
+gcc_pure
+static const char *
+SkipUriScheme(const char *uri) noexcept
 {
-	const char *auth, *slash, *at;
+	const char *const schemes[] = { "http://", "https://", "ftp://" };
+	for (auto scheme : schemes) {
+		auto result = StringAfterPrefix(uri, scheme);
+		if (result != nullptr)
+			return result;
+	}
 
-	if (memcmp(uri, "http://", 7) == 0)
-		auth = uri + 7;
-	else if (memcmp(uri, "https://", 8) == 0)
-		auth = uri + 8;
-	else if (memcmp(uri, "ftp://", 6) == 0)
-		auth = uri + 6;
-	else
+	return nullptr;
+}
+
+std::string
+uri_remove_auth(const char *uri) noexcept
+{
+	const char *auth = SkipUriScheme(uri);
+	if (auth == nullptr)
 		/* unrecognized URI */
 		return std::string();
 
-	slash = strchr(auth, '/');
+	const char *slash = strchr(auth, '/');
 	if (slash == nullptr)
 		slash = auth + strlen(auth);
 
-	at = (const char *)memchr(auth, '@', slash - auth);
+	const char *at = (const char *)memchr(auth, '@', slash - auth);
 	if (at == nullptr)
 		/* no auth info present, do nothing */
 		return std::string();
@@ -138,7 +202,7 @@ uri_remove_auth(const char *uri)
 }
 
 bool
-uri_is_child(const char *parent, const char *child)
+uri_is_child(const char *parent, const char *child) noexcept
 {
 #if !CLANG_CHECK_VERSION(3,6)
 	/* disabled on clang due to -Wtautological-pointer-compare */
@@ -153,13 +217,13 @@ uri_is_child(const char *parent, const char *child)
 
 
 bool
-uri_is_child_or_same(const char *parent, const char *child)
+uri_is_child_or_same(const char *parent, const char *child) noexcept
 {
 	return strcmp(parent, child) == 0 || uri_is_child(parent, child);
 }
 
 std::string
-uri_apply_base(const std::string &uri, const std::string &base)
+uri_apply_base(const std::string &uri, const std::string &base) noexcept
 {
 	if (uri.front() == '/') {
 		/* absolute path: replace the whole URI path in base */
