@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,6 @@
 #include "input/InputStream.hxx"
 #include "input/TextInputStream.hxx"
 #include "config/ConfigGlobal.hxx"
-#include "stdbin.h"
-#include "util/Error.hxx"
 #include "thread/Cond.hxx"
 #include "Log.hxx"
 
@@ -32,14 +30,33 @@
 #include "archive/ArchiveList.hxx"
 #endif
 
-#ifdef HAVE_GLIB
-#include <glib.h>
-#endif
+#include <stdexcept>
 
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+class GlobalInit {
+	const ScopeIOThread io_thread;
+
+public:
+	GlobalInit() {
+		config_global_init();
+#ifdef ENABLE_ARCHIVE
+		archive_plugin_init_all();
+#endif
+		input_stream_global_init();
+	}
+
+	~GlobalInit() {
+		input_stream_global_finish();
+#ifdef ENABLE_ARCHIVE
+		archive_plugin_deinit_all();
+#endif
+		config_global_finish();
+	}
+};
+
 static void
 dump_text_file(TextInputStream &is)
 {
@@ -49,86 +66,38 @@ dump_text_file(TextInputStream &is)
 }
 
 static int
-dump_input_stream(InputStream &is)
+dump_input_stream(InputStreamPtr &&is)
 {
 	{
-		TextInputStream tis(is);
+		TextInputStream tis(std::move(is));
 		dump_text_file(tis);
 	}
 
-	is.Lock();
-
-	Error error;
-	if (!is.Check(error)) {
-		LogError(error);
-		is.Unlock();
-		return EXIT_FAILURE;
-	}
-
-	is.Unlock();
+	const std::lock_guard<Mutex> protect(is->mutex);
 
+	is->Check();
 	return 0;
 }
 
 int main(int argc, char **argv)
-{
-	int ret;
-
+try {
 	if (argc != 2) {
 		fprintf(stderr, "Usage: run_input URI\n");
 		return EXIT_FAILURE;
 	}
 
-	/* initialize GLib */
-
-#ifdef HAVE_GLIB
-#if !GLIB_CHECK_VERSION(2,32,0)
-	g_thread_init(NULL);
-#endif
-#endif
-
 	/* initialize MPD */
 
-	config_global_init();
-
-	const ScopeIOThread io_thread;
-
-#ifdef ENABLE_ARCHIVE
-	archive_plugin_init_all();
-#endif
-
-	Error error;
-	if (!input_stream_global_init(error)) {
-		LogError(error);
-		return 2;
-	}
+	const GlobalInit init;
 
 	/* open the stream and dump it */
 
 	Mutex mutex;
 	Cond cond;
 
-	InputStream *is = InputStream::OpenReady(argv[1], mutex, cond, error);
-	if (is != NULL) {
-		ret = dump_input_stream(*is);
-		delete is;
-	} else {
-		if (error.IsDefined())
-			LogError(error);
-		else
-			fprintf(stderr, "input_stream::Open() failed\n");
-		ret = EXIT_FAILURE;
-	}
-
-	/* deinitialize everything */
-
-	input_stream_global_finish();
-
-#ifdef ENABLE_ARCHIVE
-	archive_plugin_deinit_all();
-#endif
-
-	config_global_finish();
-
-	return ret;
+	auto is = InputStream::OpenReady(argv[1], mutex, cond);
+	return dump_input_stream(std::move(is));
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
