@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,59 +27,35 @@
 #include "AudioParser.hxx"
 #include "AudioFormat.hxx"
 #include "pcm/PcmConvert.hxx"
-#include "config/ConfigGlobal.hxx"
 #include "util/ConstBuffer.hxx"
 #include "util/StaticFifoBuffer.hxx"
-#include "util/Error.hxx"
 #include "Log.hxx"
-#include "stdbin.h"
 
 #include <assert.h>
 #include <stddef.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <unistd.h>
 
-const char *
-config_get_string(gcc_unused enum ConfigOption option,
-		  const char *default_value)
-{
-	return default_value;
-}
-
-int main(int argc, char **argv)
-{
-	AudioFormat in_audio_format, out_audio_format;
-
+int
+main(int argc, char **argv)
+try {
 	if (argc != 3) {
 		fprintf(stderr,
 			"Usage: run_convert IN_FORMAT OUT_FORMAT <IN >OUT\n");
 		return 1;
 	}
 
-	Error error;
-	if (!audio_format_parse(in_audio_format, argv[1],
-				false, error)) {
-		LogError(error, "Failed to parse audio format");
-		return EXIT_FAILURE;
-	}
+	const auto in_audio_format = ParseAudioFormat(argv[1], false);
+	const auto out_audio_format_mask = ParseAudioFormat(argv[2], false);
 
-	AudioFormat out_audio_format_mask;
-	if (!audio_format_parse(out_audio_format_mask, argv[2],
-				true, error)) {
-		LogError(error, "Failed to parse audio format");
-		return EXIT_FAILURE;
-	}
-
-	out_audio_format = in_audio_format;
-	out_audio_format.ApplyMask(out_audio_format_mask);
+	const auto out_audio_format =
+		in_audio_format.WithMask(out_audio_format_mask);
 
 	const size_t in_frame_size = in_audio_format.GetFrameSize();
 
 	PcmConvert state;
-	if (!state.Open(in_audio_format, out_audio_format_mask, error)) {
-		LogError(error, "Failed to open PcmConvert");
-		return EXIT_FAILURE;
-	}
+	state.Open(in_audio_format, out_audio_format);
 
 	StaticFifoBuffer<uint8_t, 4096> buffer;
 
@@ -104,12 +80,7 @@ int main(int argc, char **argv)
 
 		buffer.Consume(src.size);
 
-		auto output = state.Convert({src.data, src.size}, error);
-		if (output.IsNull()) {
-			state.Close();
-			LogError(error, "Failed to convert");
-			return EXIT_FAILURE;
-		}
+		auto output = state.Convert({src.data, src.size});
 
 		gcc_unused ssize_t ignored = write(1, output.data,
 						   output.size);
@@ -118,4 +89,7 @@ int main(int argc, char **argv)
 	state.Close();
 
 	return EXIT_SUCCESS;
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
