@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,13 +26,9 @@
 #include "input/InputStream.hxx"
 #include "fs/Path.hxx"
 #include "AudioFormat.hxx"
-#include "util/Error.hxx"
 #include "Log.hxx"
-#include "stdbin.h"
 
-#ifdef HAVE_GLIB
-#include <glib.h>
-#endif
+#include <stdexcept>
 
 #include <assert.h>
 #include <unistd.h>
@@ -40,29 +36,19 @@
 #include <stdio.h>
 
 int main(int argc, char **argv)
-{
+try {
 	if (argc != 3) {
 		fprintf(stderr, "Usage: run_decoder DECODER URI >OUT\n");
 		return EXIT_FAILURE;
 	}
 
-	Decoder decoder;
+	FakeDecoder decoder;
 	const char *const decoder_name = argv[1];
 	const char *const uri = argv[2];
 
-#ifdef HAVE_GLIB
-#if !GLIB_CHECK_VERSION(2,32,0)
-	g_thread_init(NULL);
-#endif
-#endif
-
 	const ScopeIOThread io_thread;
 
-	Error error;
-	if (!input_stream_global_init(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	input_stream_global_init();
 
 	decoder_plugin_init_all();
 
@@ -75,21 +61,9 @@ int main(int argc, char **argv)
 	if (plugin->file_decode != nullptr) {
 		plugin->FileDecode(decoder, Path::FromFS(uri));
 	} else if (plugin->stream_decode != nullptr) {
-		InputStream *is =
-			InputStream::OpenReady(uri, decoder.mutex,
-					       decoder.cond, error);
-		if (is == NULL) {
-			if (error.IsDefined())
-				LogError(error);
-			else
-				fprintf(stderr, "InputStream::Open() failed\n");
-
-			return EXIT_FAILURE;
-		}
-
+		auto is = InputStream::OpenReady(uri, decoder.mutex,
+						 decoder.cond);
 		plugin->StreamDecode(decoder, *is);
-
-		delete is;
 	} else {
 		fprintf(stderr, "Decoder plugin is not usable\n");
 		return EXIT_FAILURE;
@@ -103,5 +77,8 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	return 0;
+	return EXIT_SUCCESS;
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
