@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,29 +20,21 @@
 #include "config.h"
 #include "encoder/EncoderList.hxx"
 #include "encoder/EncoderPlugin.hxx"
+#include "encoder/EncoderInterface.hxx"
+#include "encoder/ToOutputStream.hxx"
 #include "AudioFormat.hxx"
 #include "AudioParser.hxx"
-#include "config/ConfigData.hxx"
-#include "util/Error.hxx"
+#include "config/Block.hxx"
+#include "fs/io/StdioOutputStream.hxx"
 #include "Log.hxx"
-#include "stdbin.h"
+
+#include <memory>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <unistd.h>
 
-static void
-encoder_to_stdout(Encoder &encoder)
-{
-	size_t length;
-	static char buffer[32768];
-
-	while ((length = encoder_read(&encoder, buffer, sizeof(buffer))) > 0) {
-		gcc_unused ssize_t ignored = write(1, buffer, length);
-	}
-}
-
 int main(int argc, char **argv)
 {
 	const char *encoder_name;
@@ -69,52 +61,38 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	config_param param;
-	param.AddBlockParam("quality", "5.0", -1);
+	ConfigBlock block;
+	block.AddBlockParam("quality", "5.0", -1);
 
-	Error error;
-	const auto encoder = encoder_init(*plugin, param, error);
-	if (encoder == NULL) {
-		LogError(error, "Failed to initialize encoder");
-		return EXIT_FAILURE;
-	}
+	try {
+		std::unique_ptr<PreparedEncoder> p_encoder(encoder_init(*plugin, block));
 
-	/* open the encoder */
+		/* open the encoder */
 
-	AudioFormat audio_format(44100, SampleFormat::S16, 2);
-	if (argc > 2) {
-		if (!audio_format_parse(audio_format, argv[2], false, error)) {
-			LogError(error, "Failed to parse audio format");
-			return EXIT_FAILURE;
-		}
-	}
+		AudioFormat audio_format(44100, SampleFormat::S16, 2);
+		if (argc > 2)
+			audio_format = ParseAudioFormat(argv[2], false);
 
-	if (!encoder_open(encoder, audio_format, error)) {
-		LogError(error, "Failed to open encoder");
-		return EXIT_FAILURE;
-	}
+		std::unique_ptr<Encoder> encoder(p_encoder->Open(audio_format));
 
-	encoder_to_stdout(*encoder);
+		StdioOutputStream os(stdout);
 
-	/* do it */
+		EncoderToOutputStream(os, *encoder);
 
-	ssize_t nbytes;
-	while ((nbytes = read(0, buffer, sizeof(buffer))) > 0) {
-		if (!encoder_write(encoder, buffer, nbytes, error)) {
-			LogError(error, "encoder_write() failed");
-			return EXIT_FAILURE;
+		/* do it */
+
+		ssize_t nbytes;
+		while ((nbytes = read(0, buffer, sizeof(buffer))) > 0) {
+			encoder->Write(buffer, nbytes);
+			EncoderToOutputStream(os, *encoder);
 		}
 
-		encoder_to_stdout(*encoder);
-	}
+		encoder->End();
+		EncoderToOutputStream(os, *encoder);
 
-	if (!encoder_end(encoder, error)) {
-		LogError(error, "encoder_flush() failed");
+		return EXIT_SUCCESS;
+	} catch (const std::exception &e) {
+		LogError(e);
 		return EXIT_FAILURE;
 	}
-
-	encoder_to_stdout(*encoder);
-
-	encoder_close(encoder);
-	encoder_finish(encoder);
 }
