@@ -23,6 +23,7 @@
 #include "storage/Registry.hxx"
 #include "storage/StorageInterface.hxx"
 #include "storage/FileInfo.hxx"
+#include "net/Init.hxx"
 
 #include <memory>
 #include <stdexcept>
@@ -70,7 +71,12 @@ Ls(Storage &storage, const char *path)
 		char mtime_buffer[32];
 		const char *mtime = "          ";
 		if (info.mtime > 0) {
-			strftime(mtime_buffer, sizeof(mtime_buffer), "%F",
+			strftime(mtime_buffer, sizeof(mtime_buffer),
+#ifdef _WIN32
+				 "%Y-%m-%d",
+#else
+				 "%F",
+#endif
 				 gmtime(&info.mtime));
 			mtime = mtime_buffer;
 		}
@@ -95,6 +101,7 @@ try {
 	const char *const command = argv[1];
 	const char *const storage_uri = argv[2];
 
+	const ScopeNetInit net_init;
 	const ScopeIOThread io_thread;
 
 	if (strcmp(command, "ls") == 0) {
