@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,8 +28,6 @@
 #include "AudioParser.hxx"
 #include "AudioFormat.hxx"
 #include "util/ConstBuffer.hxx"
-#include "util/Error.hxx"
-#include "stdbin.h"
 #include "Log.hxx"
 
 #include <stdio.h>
@@ -37,8 +35,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-int main(int argc, char **argv)
-{
+int
+main(int argc, char **argv)
+try {
 	static char buffer[4096];
 	ssize_t nbytes;
 
@@ -47,20 +46,12 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	Error error;
 	AudioFormat audio_format(48000, SampleFormat::S16, 2);
-	if (argc > 1) {
-		if (!audio_format_parse(audio_format, argv[1], false, error)) {
-			LogError(error, "Failed to parse audio format");
-			return EXIT_FAILURE;
-		}
-	}
+	if (argc > 1)
+		audio_format = ParseAudioFormat(argv[1], false);
 
 	PcmVolume pv;
-	if (!pv.Open(audio_format.format, error)) {
-		fprintf(stderr, "%s\n", error.GetMessage());
-		return EXIT_FAILURE;
-	}
+	pv.Open(audio_format.format);
 
 	while ((nbytes = read(0, buffer, sizeof(buffer))) > 0) {
 		auto dest = pv.Apply({buffer, size_t(nbytes)});
@@ -68,4 +59,7 @@ int main(int argc, char **argv)
 	}
 
 	pv.Close();
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
