@@ -28,7 +28,7 @@ icy_parse_tag(const char *p)
 static void
 CompareTagTitle(const Tag &tag, const std::string &title)
 {
-	CPPUNIT_ASSERT_EQUAL(1u, tag.num_items);
+	CPPUNIT_ASSERT_EQUAL(uint16_t(1), tag.num_items);
 
 	const TagItem &item = *tag.items[0];
 	CPPUNIT_ASSERT_EQUAL(TAG_TITLE, item.type);
@@ -47,7 +47,7 @@ static void
 TestIcyParserEmpty(const char *input)
 {
 	Tag *tag = icy_parse_tag(input);
-	CPPUNIT_ASSERT_EQUAL(0u, tag->num_items);
+	CPPUNIT_ASSERT_EQUAL(uint16_t(0), tag->num_items);
 	delete tag;
 }
 
