@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 #include "pcm/PcmDither.hxx"
 #include "pcm/PcmUtils.hxx"
 #include "pcm/PcmBuffer.hxx"
-#include "AudioFormat.hxx"
+#include "pcm/SampleFormat.hxx"
 
 void
 PcmFormatTest::TestFormat8to16()
